"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const noRecoveryParser_1 = require("../mainParser/noRecoveryParser");
const helperTypes_1 = require("./helperTypes");
const recusiveGetTokenString_1 = require("./recusiveGetTokenString");
const types_1 = require("./types");
const BaseCstVisitor = noRecoveryParser_1.parserInstance.getBaseCstVisitorConstructorWithDefaults();
function getPosition(l) {
    return {
        startLine: l.startLine,
        startOffset: l.startOffset,
        endLine: l.endLine,
        endColumn: l.endColumn,
    };
}
class PlSqlInterpreter extends BaseCstVisitor {
    constructor() {
        super();
        this.validateVisitor();
    }
    global(ctx) {
        const globalObjects = {};
        if (ctx.createPackage && typeof ctx.createPackage.length !== 'undefined') {
            globalObjects.packages = [];
            ctx.createPackage.forEach((p) => globalObjects.packages.push(this.visit(p)));
        }
        return globalObjects;
    }
    createPackage(ctx) {
        const pkg = {};
        if (ctx.createPackageSpec && ctx.createPackageSpec[0]) {
            const ps = ctx.createPackageSpec[0];
            pkg.type = types_1.ObjectContext.spec;
            pkg.position = getPosition(ps.location);
            pkg.name = (0, recusiveGetTokenString_1.default)(ps.children.package_name);
            pkg.content = {};
            if (ps.children.variableDeclaration) {
                ps.children.variableDeclaration.forEach((vd) => {
                    const v = this.visit(vd);
                    if (v.type === helperTypes_1.VarTypes.variable) {
                        if (!pkg.content.variables) {
                            pkg.content.variables = [v.def];
                        }
                        else {
                            pkg.content.variables.push(v.def);
                        }
                    }
                    else if (v.type === helperTypes_1.VarTypes.exception) {
                        if (!pkg.content.exceptions) {
                            pkg.content.exceptions = [v.def];
                        }
                        else {
                            pkg.content.exceptions.push(v.def);
                        }
                    }
                    else {
                        throw new Error(`Unhandled variable type ${v.type}`);
                    }
                });
            }
            if (ps.children.objectDeclaration) {
                pkg.content.objects = ps.children.objectDeclaration.map((p) => this.visit(p));
            }
        }
        else if (ctx.createPackageBody && ctx.createPackageBody[0]) {
            const pb = ctx.createPackageBody[0];
            pkg.type = types_1.ObjectContext.body;
            pkg.position = getPosition(pb.location);
            pkg.name = (0, recusiveGetTokenString_1.default)(pb.children.package_name);
            pkg.content = {};
            pkg.content.objects = [];
            if (pb.children.variableDeclaration) {
                pb.children.variableDeclaration.forEach((vd) => {
                    const v = this.visit(vd);
                    if (v.type === helperTypes_1.VarTypes.variable) {
                        if (!pkg.content.variables) {
                            pkg.content.variables = [v.def];
                        }
                        else {
                            pkg.content.variables.push(v.def);
                        }
                    }
                    else if (v.type === helperTypes_1.VarTypes.exception) {
                        if (!pkg.content.exceptions) {
                            pkg.content.exceptions = [v.def];
                        }
                        else {
                            pkg.content.exceptions.push(v.def);
                        }
                    }
                    else {
                        throw new Error(`Unhandled variable type ${v.type}`);
                    }
                });
            }
            if (pb.children.objectDeclaration) {
                pkg.content.objects = pb.children.objectDeclaration.map((p) => this.visit(p));
            }
            if (pb.children.packageObjSpec) {
                pb.children.packageObjSpec.forEach((p) => pkg.content.objects?.push(this.visit(p)));
            }
            if (pb.children.typeDefiniton) {
                pkg.content.types = pb.children.typeDefiniton.map((p) => {
                    const t = this.visit(p);
                    if (p.location) {
                        t.position = getPosition(p.location);
                    }
                    return t;
                });
            }
        }
        return pkg;
    }
    variableDeclaration(ctx) {
        if (ctx.standardVariableDeclaration?.[0]) {
            const v = {};
            const sv = ctx.standardVariableDeclaration[0];
            v.position = getPosition(sv.location);
            v.name = (0, recusiveGetTokenString_1.default)(sv.children.variable_name);
            v.constant = !!sv.children.ConstantKw;
            if (sv.children.variableSpec) {
                v.type = (0, recusiveGetTokenString_1.default)(sv.children.variableSpec);
            }
            if (sv.children.value) {
                v.value = this.visit(sv.children.value);
            }
            return { type: helperTypes_1.VarTypes.variable, def: v };
        }
        else if (ctx.exceptionDeclaration?.[0]) {
            const e = {};
            e.position = getPosition(ctx.exceptionDeclaration[0].location);
            e.name = (0, recusiveGetTokenString_1.default)(ctx.exceptionDeclaration[0].children.exception_name);
            return { type: helperTypes_1.VarTypes.exception, def: e };
        }
        return null;
    }
    value(ctx) {
        return (0, recusiveGetTokenString_1.default)(ctx);
    }
    typeDefiniton(ctx) {
        const type = {};
        type.type = ctx.type_table_type ? types_1.TypeClass.table : types_1.TypeClass.record;
        if (ctx?.type_name) {
            type.name = (0, recusiveGetTokenString_1.default)(ctx.type_name);
        }
        if (ctx.type_table_type) {
            type.tableOf = (0, recusiveGetTokenString_1.default)(ctx.type_table_type);
        }
        if (ctx.type_table_index) {
            type.tableIndex = (0, recusiveGetTokenString_1.default)(ctx.type_table_index);
        }
        if (ctx.type_record_variable_name?.length &&
            ctx.type_record_variable_name.length > 0) {
            type.recordFields = [];
            for (let i = 0; i < ctx.type_record_variable_name.length; i++) {
                const field = {};
                field.name = (0, recusiveGetTokenString_1.default)(ctx.type_record_variable_name[i]);
                field.type = (0, recusiveGetTokenString_1.default)(ctx.type_record_variable_type[i]);
                type.recordFields.push(field);
            }
        }
        return type;
    }
    objectDeclaration(ctx) {
        if (ctx.funcSpec && ctx.funcSpec[0]) {
            const fc = this.visit(ctx.funcSpec[0]);
            fc.position = getPosition(ctx.funcSpec[0].location);
            return fc;
        }
        if (ctx.procSpec && ctx.procSpec[0]) {
            const prc = this.visit(ctx.procSpec[0]);
            prc.position = getPosition(ctx.procSpec[0].location);
            return prc;
        }
    }
    funcSpec(ctx) {
        const obj = {};
        obj.type = types_1.ObjectType.fc;
        obj.context = types_1.ObjectContext.spec;
        //obj.position = getPosition(ctx.location);
        if (ctx?.function_name) {
            obj.name = (0, recusiveGetTokenString_1.default)(ctx.function_name);
        }
        if (ctx?.argumentList) {
            obj.arguments = this.visit(ctx.argumentList);
        }
        if (ctx?.dataType) {
            obj.return = (0, recusiveGetTokenString_1.default)(ctx.dataType);
        }
        else if (ctx?.objType) {
            obj.return = (0, recusiveGetTokenString_1.default)(ctx.objType);
        }
        obj.deterministic = !!ctx?.DeterministicKw;
        obj.resultCache = !!ctx?.ResultCacheKw;
        obj.pipelined = !!ctx?.PipelinedKw;
        return obj;
    }
    procSpec(ctx) {
        const obj = {};
        obj.type = types_1.ObjectType.prc;
        obj.context = types_1.ObjectContext.spec;
        // obj.position = getPosition(ctx.location);
        if (ctx?.procedure_name) {
            obj.name = (0, recusiveGetTokenString_1.default)(ctx.procedure_name);
        }
        if (ctx?.argumentList) {
            obj.arguments = this.visit(ctx.argumentList);
        }
        return obj;
    }
    packageObjSpec(ctx) {
        if (ctx.funcBody && ctx.funcBody[0] && ctx.funcBody[0].children.funcSpec) {
            const fc = this.visit(ctx.funcBody[0].children.funcSpec);
            fc.context = types_1.ObjectContext.body;
            fc.position = getPosition(ctx.funcBody[0].location);
            return fc;
        }
        else if (ctx.procBody &&
            ctx.procBody[0] &&
            ctx.procBody[0].children.procSpec) {
            const prc = this.visit(ctx.procBody[0].children.procSpec);
            prc.context = types_1.ObjectContext.body;
            prc.position = getPosition(ctx.procBody[0].location);
            return prc;
        }
    }
    argumentList(ctx) {
        const args = [];
        ctx.argument.forEach((arg) => {
            const a = {};
            a.name = (0, recusiveGetTokenString_1.default)(arg.children.Identifier);
            if (arg.children.dataType) {
                a.dataType = (0, recusiveGetTokenString_1.default)(arg.children.dataType);
            }
            else if (arg.children.objType) {
                a.dataType = (0, recusiveGetTokenString_1.default)(arg.children.objType);
            }
            if (!arg.children.inOut) {
                a.direction = types_1.ArgumentDirection.in;
            }
            else if (arg.children.inOut[0].children.InKw &&
                arg.children.inOut[0].children.OutKw) {
                a.direction = types_1.ArgumentDirection.inOut;
            }
            else if (arg.children.inOut[0].children.InKw) {
                a.direction = types_1.ArgumentDirection.in;
            }
            else {
                a.direction = types_1.ArgumentDirection.out;
            }
            args.push(a);
        });
        return args;
    }
}
const plSqlInterpreter = new PlSqlInterpreter();
exports.default = plSqlInterpreter;
//# sourceMappingURL=index.js.map