"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chevrotain_1 = require("chevrotain");
const tokens_1 = require("../tokenDictionary/tokens");
class PlSqlParser extends chevrotain_1.CstParser {
    constructor({ recover }) {
        super(tokens_1.tokenVocabulary, {
            recoveryEnabled: recover,
            nodeLocationTracking: 'full',
        });
        // eslint-disable-next-line @typescript-eslint/no-this-alias
        const $ = this;
        $.logStack = (stack = $.CST_STACK) => {
            const msg = [];
            stack.forEach((item) => {
                const obj = {
                    name: item.name,
                    children: JSON.stringify(item.children),
                };
                msg.push(obj);
            });
            return msg;
        };
        $.RULE('global', () => {
            $.MANY(() => {
                $.OR({
                    MAX_LOOKAHEAD: 4,
                    DEF: [
                        // plsql
                        {
                            ALT: () => {
                                $.OR1([
                                    { ALT: () => $.SUBRULE($.block) },
                                    { ALT: () => $.SUBRULE($.createPackage) },
                                ]);
                                $.OPTION(() => {
                                    $.CONSUME(tokens_1.tokenVocabulary.Slash);
                                });
                            },
                        },
                        { ALT: () => $.SUBRULE($.createSequenceStatement) },
                        { ALT: () => $.SUBRULE($.createViewStatement) },
                    ],
                });
            });
        });
        $.RULE('createPackage', () => {
            $.OR({
                MAX_LOOKAHEAD: 5,
                DEF: [
                    { ALT: () => $.SUBRULE($.createPackageSpec) },
                    { ALT: () => $.SUBRULE($.createPackageBody) },
                ],
            });
        });
        $.RULE('typeDefiniton', () => {
            $.CONSUME(tokens_1.tokenVocabulary.Type);
            $.CONSUME(tokens_1.tokenVocabulary.Identifier, { LABEL: 'type_name' });
            $.CONSUME(tokens_1.tokenVocabulary.IsKw);
            $.OR([
                {
                    // table type
                    ALT: () => {
                        $.CONSUME(tokens_1.tokenVocabulary.TableKw);
                        $.CONSUME(tokens_1.tokenVocabulary.OfKw);
                        $.SUBRULE($.variableSpec, { LABEL: 'type_table_type' });
                        $.OPTION(() => {
                            $.CONSUME(tokens_1.tokenVocabulary.IndexKw);
                            $.CONSUME(tokens_1.tokenVocabulary.ByKw);
                            $.SUBRULE1($.variableSpec, { LABEL: 'type_table_index' });
                        });
                    },
                },
                {
                    // record type
                    ALT: () => {
                        $.CONSUME(tokens_1.tokenVocabulary.RecordKw);
                        $.CONSUME(tokens_1.tokenVocabulary.OpenBracket);
                        $.AT_LEAST_ONE_SEP({
                            SEP: tokens_1.tokenVocabulary.Comma,
                            DEF: () => {
                                $.CONSUME1(tokens_1.tokenVocabulary.Identifier, {
                                    LABEL: 'type_record_variable_name',
                                });
                                $.SUBRULE2($.variableSpec, {
                                    LABEL: 'type_record_variable_type',
                                });
                            },
                        });
                        $.CONSUME(tokens_1.tokenVocabulary.ClosingBracket);
                    },
                },
            ]);
            $.CONSUME(tokens_1.tokenVocabulary.Semicolon);
        });
        $.RULE('block', () => {
            $.OPTION(() => {
                $.CONSUME(tokens_1.tokenVocabulary.Declare);
                $.MANY(() => {
                    $.OR([
                        { ALT: () => $.SUBRULE($.packageObjSpec) },
                        { ALT: () => $.SUBRULE($.variableDeclaration) },
                        { ALT: () => $.SUBRULE($.typeDefiniton) },
                    ]);
                });
            });
            $.SUBRULE($.beginClause);
            $.MANY2(() => {
                $.SUBRULE($.statement);
            });
            $.OPTION2(() => {
                $.SUBRULE($.exceptionBlock);
            });
            $.SUBRULE($.endClause);
            $.SUBRULE($.semicolon);
        });
        $.RULE('beginClause', () => {
            $.CONSUME(tokens_1.tokenVocabulary.Begin);
        });
        $.RULE('endClause', () => {
            $.CONSUME(tokens_1.tokenVocabulary.End);
        });
        $.RULE('throwException', () => {
            $.CONSUME(tokens_1.tokenVocabulary.RaiseKw); // raise
            $.OPTION(() => {
                $.SUBRULE($.dottedIdentifier); // exception_name or var.exception_type
            });
            $.SUBRULE($.semicolon);
        });
        $.RULE('caseGlobalExpression', () => {
            $.SUBRULE($.value);
            $.AT_LEAST_ONE(() => {
                $.CONSUME(tokens_1.tokenVocabulary.WhenKw);
                $.SUBRULE1($.value);
                $.CONSUME(tokens_1.tokenVocabulary.Then);
                $.MANY(() => {
                    $.SUBRULE($.statement);
                });
            });
        });
        $.RULE('caseWithoutGlobalExpression', () => {
            $.AT_LEAST_ONE(() => {
                $.CONSUME(tokens_1.tokenVocabulary.WhenKw);
                $.SUBRULE($.condition);
                $.CONSUME(tokens_1.tokenVocabulary.Then);
                $.MANY(() => {
                    $.SUBRULE($.statement);
                });
            });
        });
        $.RULE('caseStatement', () => {
            $.CONSUME(tokens_1.tokenVocabulary.CaseKw);
            $.OR([
                { ALT: () => $.SUBRULE($.caseGlobalExpression) },
                { ALT: () => $.SUBRULE($.caseWithoutGlobalExpression) },
            ]);
            $.OPTION(() => {
                $.CONSUME(tokens_1.tokenVocabulary.Else);
                $.MANY(() => {
                    $.SUBRULE2($.statement);
                });
            });
            $.CONSUME(tokens_1.tokenVocabulary.End);
            $.OPTION2(() => {
                $.CONSUME2(tokens_1.tokenVocabulary.CaseKw);
            });
            $.SUBRULE($.semicolon);
        });
        $.RULE('sqlCaseGlobalExpression', () => {
            $.SUBRULE($.value);
            $.AT_LEAST_ONE(() => {
                $.CONSUME(tokens_1.tokenVocabulary.WhenKw);
                $.SUBRULE1($.value);
                $.CONSUME(tokens_1.tokenVocabulary.Then);
                $.SUBRULE2($.value);
            });
        });
        $.RULE('sqlCaseWithoutGlobalExpression', () => {
            $.AT_LEAST_ONE(() => {
                $.CONSUME(tokens_1.tokenVocabulary.WhenKw);
                $.SUBRULE($.condition);
                $.CONSUME(tokens_1.tokenVocabulary.Then);
                $.SUBRULE($.value);
            });
        });
        $.RULE('sqlCaseStatement', () => {
            $.CONSUME(tokens_1.tokenVocabulary.CaseKw);
            $.OR([
                { ALT: () => $.SUBRULE($.sqlCaseGlobalExpression) },
                { ALT: () => $.SUBRULE($.sqlCaseWithoutGlobalExpression) },
            ]);
            $.OPTION(() => {
                $.CONSUME(tokens_1.tokenVocabulary.Else);
                $.MANY(() => {
                    $.SUBRULE($.value);
                });
            });
            $.CONSUME(tokens_1.tokenVocabulary.End);
        });
        $.RULE('pipeRowStatement', () => {
            $.CONSUME(tokens_1.tokenVocabulary.PipeRowKw);
            $.CONSUME(tokens_1.tokenVocabulary.OpenBracket);
            $.SUBRULE($.value);
            // $.OPTION(() => {
            $.CONSUME(tokens_1.tokenVocabulary.ClosingBracket);
            //});
            $.SUBRULE($.semicolon);
        });
        $.RULE('gotoLabelDefinitionStatement', () => {
            $.CONSUME(tokens_1.tokenVocabulary.Smaller); // <
            $.CONSUME1(tokens_1.tokenVocabulary.Smaller); // <
            $.CONSUME(tokens_1.tokenVocabulary.Identifier); // my_goto_label
            $.CONSUME(tokens_1.tokenVocabulary.Bigger); // >
            $.CONSUME1(tokens_1.tokenVocabulary.Bigger); // >
        });
        $.RULE('gotoStatement', () => {
            $.CONSUME(tokens_1.tokenVocabulary.GotoKw);
            $.CONSUME(tokens_1.tokenVocabulary.Identifier);
            $.SUBRULE($.semicolon);
        });
        $.RULE('statement', () => {
            $.OR([
                { GATE: $.BACKTRACK($.assignment), ALT: () => $.SUBRULE($.assignment) },
                ...($.XstatementOr ??
                    ($.XstatementOr = [
                        { ALT: () => $.SUBRULE($.nullStatement) },
                        { ALT: () => $.SUBRULE($.exitStatement) },
                        { ALT: () => $.SUBRULE($.ifStatement) },
                        { ALT: () => $.SUBRULE($.compilerIfStatement) },
                        { ALT: () => $.SUBRULE($.caseStatement) },
                        {
                            ALT: () => $.SUBRULE($.functionCallSemicolon),
                        },
                        { ALT: () => $.SUBRULE($.queryStatement) },
                        { ALT: () => $.SUBRULE($.dmlStatement) },
                        { ALT: () => $.SUBRULE($.forallStatement) },
                        { ALT: () => $.SUBRULE($.transactionStatement) },
                        { ALT: () => $.SUBRULE($.forLoop) },
                        { ALT: () => $.SUBRULE($.whileLoop) },
                        { ALT: () => $.SUBRULE($.exitLoop) },
                        { ALT: () => $.SUBRULE($.returnStatement) },
                        { ALT: () => $.SUBRULE($.block) },
                        { ALT: () => $.SUBRULE($.dynamicSqlStatement) },
                        { ALT: () => $.SUBRULE($.throwException) },
                        { ALT: () => $.SUBRULE($.pipeRowStatement) },
                        { ALT: () => $.SUBRULE($.gotoLabelDefinitionStatement) },
                        { ALT: () => $.SUBRULE($.gotoStatement) },
                    ])),
            ]);
        });
        $.RULE('exitStatement', () => {
            $.CONSUME(tokens_1.tokenVocabulary.ExitKw);
            $.OPTION(() => {
                $.CONSUME(tokens_1.tokenVocabulary.WhenKw);
                $.SUBRULE($.chainedConditions);
            });
            $.SUBRULE($.semicolon);
        });
        $.RULE('nullStatement', () => {
            $.CONSUME(tokens_1.tokenVocabulary.Null);
            $.SUBRULE($.semicolon);
        });
        $.RULE('chainedConditions', () => {
            $.AT_LEAST_ONE_SEP({
                SEP: tokens_1.tokenVocabulary.AndOr,
                DEF: () => $.SUBRULE($.condition),
            });
        });
        $.RULE('chainedConditionsWhere', () => {
            $.AT_LEAST_ONE_SEP({
                SEP: tokens_1.tokenVocabulary.AndOr,
                DEF: () => $.SUBRULE($.conditionWhere),
            });
        });
        $.RULE('ifCondition', () => {
            $.SUBRULE($.chainedConditions);
            $.CONSUME(tokens_1.tokenVocabulary.Then); // Then
        });
        $.RULE('compilerIfCondition', () => {
            $.SUBRULE($.chainedConditions);
            $.CONSUME(tokens_1.tokenVocabulary.Dollar); // $
            $.CONSUME(tokens_1.tokenVocabulary.Then); // Then
        });
        $.RULE('ifStatement', () => {
            $.CONSUME(tokens_1.tokenVocabulary.If); // if
            $.SUBRULE($.ifCondition);
            $.MANY(() => {
                $.SUBRULE2($.statement); // ...
            });
            $.OPTION(() => {
                $.MANY2(() => {
                    $.CONSUME(tokens_1.tokenVocabulary.Elsif); // elsif
                    $.SUBRULE2($.ifCondition);
                    $.MANY3(() => {
                        $.SUBRULE4($.statement); // ...
                    });
                });
            });
            $.OPTION2(() => {
                $.CONSUME(tokens_1.tokenVocabulary.Else); // else
                $.MANY4(() => {
                    $.SUBRULE5($.statement); // ...
                });
            });
            $.CONSUME(tokens_1.tokenVocabulary.End); // end
            $.CONSUME2(tokens_1.tokenVocabulary.If); // if
            $.SUBRULE($.semicolon);
        });
        $.RULE('compileErrorStatement', () => {
            $.CONSUME(tokens_1.tokenVocabulary.CompileErrorKw);
            $.AT_LEAST_ONE(() => {
                $.SUBRULE($.value);
            });
            $.CONSUME3(tokens_1.tokenVocabulary.Dollar); // $
            $.CONSUME(tokens_1.tokenVocabulary.End); // end
        });
        $.RULE('compilerIfStatement', () => {
            $.CONSUME(tokens_1.tokenVocabulary.Dollar); // $
            $.CONSUME(tokens_1.tokenVocabulary.If); // if
            $.SUBRULE($.compilerIfCondition);
            $.MANY(() => {
                $.OR([
                    { ALT: () => $.SUBRULE($.statement) },
                    { ALT: () => $.SUBRULE($.compileErrorStatement) },
                ]);
            });
            $.OPTION(() => {
                $.MANY2(() => {
                    $.CONSUME1(tokens_1.tokenVocabulary.Dollar); // $
                    $.CONSUME(tokens_1.tokenVocabulary.Elsif); // elsif
                    $.SUBRULE2($.compilerIfCondition);
                    $.MANY3(() => {
                        $.OR2([
                            { ALT: () => $.SUBRULE2($.statement) },
                            { ALT: () => $.SUBRULE2($.compileErrorStatement) },
                        ]);
                    });
                });
            });
            $.OPTION2(() => {
                $.CONSUME2(tokens_1.tokenVocabulary.Dollar); // $
                $.CONSUME(tokens_1.tokenVocabulary.Else); // else
                $.MANY4(() => {
                    $.OR3([
                        { ALT: () => $.SUBRULE3($.statement) },
                        { ALT: () => $.SUBRULE3($.compileErrorStatement) },
                    ]);
                });
            });
            $.CONSUME3(tokens_1.tokenVocabulary.Dollar); // $
            $.CONSUME(tokens_1.tokenVocabulary.End); // end
            // no if keyword
            // no semicolon
        });
        $.RULE('relationalOperators', () => {
            $.OR($.XrelOpOr ??
                ($.XrelOpOr = [
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.Equals) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.UnEquals1) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.UnEquals2) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.UnEquals3) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.BiggerEquals) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.Bigger) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.SmallerEquals) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.Smaller) },
                    {
                        ALT: () => {
                            $.OPTION(() => {
                                $.CONSUME(tokens_1.tokenVocabulary.NotKw);
                            });
                            $.CONSUME(tokens_1.tokenVocabulary.LikeKw);
                        },
                    },
                ]));
        });
        $.RULE('conditionsInBrackets', () => {
            $.OPTION(() => $.CONSUME(tokens_1.tokenVocabulary.NotKw));
            $.CONSUME(tokens_1.tokenVocabulary.OpenBracket);
            $.AT_LEAST_ONE_SEP({
                SEP: tokens_1.tokenVocabulary.AndOr,
                DEF: () => {
                    $.SUBRULE($.condition);
                },
            });
            $.CONSUME(tokens_1.tokenVocabulary.ClosingBracket);
        });
        $.RULE('conditionsInBracketsWhere', () => {
            $.OPTION(() => $.CONSUME(tokens_1.tokenVocabulary.NotKw));
            $.CONSUME(tokens_1.tokenVocabulary.OpenBracket);
            $.AT_LEAST_ONE_SEP({
                SEP: tokens_1.tokenVocabulary.AndOr,
                DEF: () => {
                    $.SUBRULE($.conditionWhere);
                },
            });
            $.CONSUME(tokens_1.tokenVocabulary.ClosingBracket);
        });
        $.RULE('nullCheck', () => {
            $.CONSUME(tokens_1.tokenVocabulary.IsKw);
            $.OPTION(() => {
                $.CONSUME(tokens_1.tokenVocabulary.NotKw);
            });
            $.CONSUME(tokens_1.tokenVocabulary.Null, { LABEL: 'rhs' });
        });
        $.RULE('singleCondition', () => {
            $.OPTION(() => {
                $.CONSUME(tokens_1.tokenVocabulary.NotKw);
            });
            $.SUBRULE($.valueWithoutOperators, { LABEL: 'lhs' });
            $.OPTION2(() => {
                $.OR([
                    {
                        ALT: () => {
                            $.SUBRULE($.relationalOperators);
                            $.SUBRULE2($.value, { LABEL: 'rhs' });
                        },
                    },
                    {
                        ALT: () => $.SUBRULE($.nullCheck),
                    },
                    {
                        // in (...)
                        ALT: () => {
                            $.OPTION4(() => {
                                $.CONSUME3(tokens_1.tokenVocabulary.NotKw);
                            });
                            $.CONSUME(tokens_1.tokenVocabulary.InKw);
                            $.OR2([
                                { ALT: () => $.SUBRULE($.valueInBrackets, { LABEL: 'rhs' }) },
                                {
                                    ALT: () => {
                                        $.SUBRULE($.queryInBrackets);
                                    },
                                },
                            ]);
                        },
                    },
                    {
                        ALT: () => {
                            $.CONSUME(tokens_1.tokenVocabulary.MemberOfKw);
                            $.SUBRULE1($.valueInBrackets, { LABEL: 'rhs' });
                        },
                    },
                ]);
            });
        });
        // allow for subquery here
        $.RULE('singleConditionWhere', () => {
            $.OPTION(() => {
                $.CONSUME(tokens_1.tokenVocabulary.NotKw);
            });
            $.OR([
                // allow subquery here
                {
                    ALT: () => {
                        $.SUBRULE($.queryInBrackets);
                    },
                },
                { ALT: () => $.SUBRULE($.valueWithoutOperators, { LABEL: 'lhs' }) },
            ]);
            $.OPTION2(() => {
                $.OR5([
                    {
                        ALT: () => {
                            $.SUBRULE($.relationalOperators);
                            $.OR3([
                                // allow subquery here
                                {
                                    ALT: () => {
                                        $.SUBRULE1($.queryInBrackets);
                                    },
                                },
                                {
                                    ALT: () => $.SUBRULE1($.valueWithoutOperators, { LABEL: 'rhs' }),
                                },
                            ]);
                        },
                    },
                    {
                        ALT: () => $.SUBRULE($.nullCheck),
                    },
                    {
                        // in (...)
                        ALT: () => {
                            $.OPTION4(() => {
                                $.CONSUME3(tokens_1.tokenVocabulary.NotKw);
                            });
                            $.CONSUME(tokens_1.tokenVocabulary.InKw);
                            $.OR4([
                                { ALT: () => $.SUBRULE($.valueInBrackets, { LABEL: 'rhs' }) },
                                {
                                    ALT: () => {
                                        $.SUBRULE2($.queryInBrackets);
                                    },
                                },
                            ]);
                        },
                    },
                    {
                        ALT: () => {
                            $.CONSUME(tokens_1.tokenVocabulary.MemberOfKw);
                            $.SUBRULE1($.valueInBrackets, { LABEL: 'rhs' });
                        },
                    },
                ]);
            });
        });
        $.RULE('condition', () => {
            $.OR({
                DEF: [
                    {
                        GATE: $.BACKTRACK($.conditionsInBrackets),
                        ALT: () => $.SUBRULE($.conditionsInBrackets),
                    },
                    { ALT: () => $.SUBRULE($.singleCondition) },
                ],
            });
        });
        $.RULE('conditionWhere', () => {
            $.OR({
                DEF: [
                    {
                        GATE: $.BACKTRACK($.conditionsInBracketsWhere),
                        ALT: () => $.SUBRULE($.conditionsInBracketsWhere),
                    },
                    { ALT: () => $.SUBRULE($.singleConditionWhere) },
                ],
            });
        });
        $.RULE('createPackageStatement', () => {
            $.CONSUME(tokens_1.tokenVocabulary.CreateKw); // create
            $.OPTION(() => {
                $.CONSUME(tokens_1.tokenVocabulary.OrKw); // or
                $.CONSUME(tokens_1.tokenVocabulary.ReplaceKw); // replace
            });
            $.CONSUME(tokens_1.tokenVocabulary.PackageKw); // package
        });
        $.RULE('createPackageSpec', () => {
            $.SUBRULE($.createPackageStatement); // create (or replace) package
            $.SUBRULE($.dottedIdentifier, { LABEL: 'package_name' }); // pkg_name | schema_name.pkg_name
            $.OPTION(() => {
                $.CONSUME(tokens_1.tokenVocabulary.AuthidKw); // authid
                $.OR([
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.DefinerKw) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.CurrentUserKw) }, // current_user
                ]);
            });
            $.CONSUME(tokens_1.tokenVocabulary.AsIs); // as
            $.MANY(() => {
                $.OR2([
                    { ALT: () => $.SUBRULE($.objectDeclaration) },
                    { ALT: () => $.SUBRULE($.variableDeclaration) },
                    { ALT: () => $.SUBRULE($.typeDefiniton) },
                    { ALT: () => $.SUBRULE($.pragmaStatement) },
                ]);
            });
            $.CONSUME(tokens_1.tokenVocabulary.End); // end
            $.OPTION2(() => {
                $.CONSUME2(tokens_1.tokenVocabulary.Identifier); // pkg_name
            });
            $.SUBRULE($.semicolon); // ;
        });
        $.RULE('createPackageBody', () => {
            $.SUBRULE($.createPackageStatement); // create (or replace) package
            $.CONSUME(tokens_1.tokenVocabulary.BodyKw); // body
            $.SUBRULE($.dottedIdentifier, { LABEL: 'package_name' }); // pkg_name | schema_name.pkg_name
            $.CONSUME(tokens_1.tokenVocabulary.AsIs);
            $.MANY(() => {
                $.OR2([
                    { ALT: () => $.SUBRULE($.packageObjSpec) },
                    { ALT: () => $.SUBRULE($.variableDeclaration) },
                    { ALT: () => $.SUBRULE($.typeDefiniton) },
                    { ALT: () => $.SUBRULE($.pragmaStatement) },
                ]);
            });
            $.CONSUME(tokens_1.tokenVocabulary.End); // end
            $.OPTION2(() => {
                $.CONSUME2(tokens_1.tokenVocabulary.Identifier); // pkg_name
            });
            $.SUBRULE($.semicolon); // ;
        });
        // TODO Add more to the pkg obj spec: http://cui.unige.ch/isi/bnf/PLSQL21/package_obj_spec.html
        $.RULE('packageObjSpec', () => {
            $.OR([
                { ALT: () => $.SUBRULE($.funcBody) },
                { ALT: () => $.SUBRULE($.procBody) },
            ]);
        });
        $.RULE('returnStatement', () => {
            $.CONSUME(tokens_1.tokenVocabulary.ReturnKw); // return
            $.OPTION(() => {
                $.SUBRULE($.value); // 3, l_var ...
            });
            $.SUBRULE($.semicolon); // ;
        });
        $.RULE('funcBody', () => {
            $.SUBRULE($.funcSpec);
            $.CONSUME(tokens_1.tokenVocabulary.AsIs); // as
            $.MANY(() => {
                $.OR([
                    { ALT: () => $.SUBRULE($.variableDeclaration) },
                    { ALT: () => $.SUBRULE($.pragmaStatement) },
                    { ALT: () => $.SUBRULE($.packageObjSpec) },
                    { ALT: () => $.SUBRULE($.typeDefiniton) },
                ]);
            });
            $.CONSUME(tokens_1.tokenVocabulary.Begin); // begin
            $.MANY2(() => {
                $.SUBRULE($.statement);
            });
            $.OPTION(() => {
                $.SUBRULE($.exceptionBlock); // exception ...
            });
            $.CONSUME(tokens_1.tokenVocabulary.End); // end
            $.OPTION2(() => {
                $.CONSUME2(tokens_1.tokenVocabulary.Identifier); // my_fnc
            });
            $.SUBRULE2($.semicolon); // ;
        });
        $.RULE('procBody', () => {
            $.SUBRULE($.procSpec);
            $.CONSUME(tokens_1.tokenVocabulary.AsIs); // as
            $.MANY(() => {
                $.OR([
                    { ALT: () => $.SUBRULE($.variableDeclaration) },
                    { ALT: () => $.SUBRULE($.pragmaStatement) },
                    { ALT: () => $.SUBRULE($.packageObjSpec) },
                    { ALT: () => $.SUBRULE($.typeDefiniton) },
                ]);
            });
            $.CONSUME(tokens_1.tokenVocabulary.Begin); // begin
            $.MANY2(() => {
                $.SUBRULE($.statement);
            });
            $.OPTION(() => {
                $.SUBRULE($.exceptionBlock); // exception ...
            });
            $.CONSUME(tokens_1.tokenVocabulary.End); // end
            $.OPTION2(() => {
                $.CONSUME2(tokens_1.tokenVocabulary.Identifier); // my_pkg
            });
            $.SUBRULE2($.semicolon); // ;
        });
        $.RULE('cursorDeclaration', () => {
            $.CONSUME(tokens_1.tokenVocabulary.CursorKw); // cursor
            $.CONSUME(tokens_1.tokenVocabulary.Identifier, { LABEL: 'cursor_name' }); // my_cursor
            $.CONSUME(tokens_1.tokenVocabulary.IsKw); // is
            $.SUBRULE($.query);
            $.CONSUME(tokens_1.tokenVocabulary.Semicolon); // ;
        });
        $.RULE('rawDeclaration', () => {
            $.CONSUME(tokens_1.tokenVocabulary.DtypeRaw); // varchar2
            $.CONSUME(tokens_1.tokenVocabulary.OpenBracket); // (
            $.CONSUME(tokens_1.tokenVocabulary.Integer); // 32
            $.CONSUME(tokens_1.tokenVocabulary.ClosingBracket); // )
        });
        $.RULE('exceptionDeclaration', () => {
            $.CONSUME(tokens_1.tokenVocabulary.Identifier, { LABEL: 'exception_name' });
            $.CONSUME(tokens_1.tokenVocabulary.ExceptionKw);
            $.CONSUME(tokens_1.tokenVocabulary.Semicolon);
        });
        $.RULE('variableSpec', () => {
            $.OR($.XvarDeclarationOr ??
                ($.XvarDeclarationOr = [
                    { ALT: () => $.SUBRULE($.numberDeclaration) },
                    { ALT: () => $.SUBRULE($.stringDeclaration) },
                    { ALT: () => $.SUBRULE($.rawDeclaration) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.DtypePlsIteger) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.DtypeBoolean) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.DtypeDate) },
                    { ALT: () => $.SUBRULE($.timestampDeclaration) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.JsonDtypes) },
                    { ALT: () => $.SUBRULE($.objType) }, // custom types e.g. rowtype
                ]));
        });
        // TODO make sure return is only called in functions
        $.RULE('standardVariableDeclaration', () => {
            // follow pattern ident (constant) type...
            $.CONSUME(tokens_1.tokenVocabulary.Identifier, { LABEL: 'variable_name' }); // l_row
            $.OPTION(() => {
                $.CONSUME(tokens_1.tokenVocabulary.ConstantKw);
            });
            $.SUBRULE($.variableSpec);
            $.OPTION2(() => {
                $.OR1([
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.Assignment) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.DefaultKw) }, // default
                ]);
                $.SUBRULE($.value);
            });
            $.SUBRULE($.semicolon);
        });
        $.RULE('jsonDtypeValue', () => {
            $.CONSUME(tokens_1.tokenVocabulary.JsonDtypes); // JSON_OBJECT_T / JSON_ARRAY_T ...
            $.CONSUME(tokens_1.tokenVocabulary.OpenBracket); // (
            $.OPTION(() => {
                $.OR([
                    { ALT: () => $.SUBRULE($.functionCall), IGNORE_AMBIGUITIES: true },
                    { ALT: () => $.SUBRULE($.stringExpression) }, // '{"json": "data"}'
                ]);
            });
            $.CONSUME(tokens_1.tokenVocabulary.ClosingBracket); // )
        });
        $.RULE('variableDeclaration', () => {
            $.OR([
                { ALT: () => $.SUBRULE($.standardVariableDeclaration) },
                // { ALT: () => $.SUBRULE($.objectDeclaration) },
                { ALT: () => $.SUBRULE($.cursorDeclaration) },
                { ALT: () => $.SUBRULE($.exceptionDeclaration) },
            ]);
        });
        $.RULE('objType', () => {
            $.SUBRULE($.dottedIdentifier);
            $.OPTION3(() => {
                $.CONSUME(tokens_1.tokenVocabulary.Percent); // %
                $.OR([
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.Rowtype) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.Type) }, // type
                ]);
            });
        });
        $.RULE('pragmaStatement', () => {
            $.CONSUME(tokens_1.tokenVocabulary.PragmaKw);
            $.OR([
                { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.AutonomousTransactionKw) },
                {
                    ALT: () => {
                        $.CONSUME(tokens_1.tokenVocabulary.ExceptionInitKw);
                        $.SUBRULE($.valueInBrackets);
                    },
                },
                { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.RestrictReferencesKw) },
                { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.SeriallyReusableKw) },
            ]);
            $.SUBRULE($.semicolon);
        });
        $.RULE('objectDeclaration', () => {
            $.OR([
                {
                    ALT: () => {
                        $.SUBRULE($.funcSpec);
                        $.SUBRULE($.semicolon);
                    },
                },
                {
                    ALT: () => {
                        $.SUBRULE($.procSpec);
                        $.SUBRULE2($.semicolon);
                    },
                },
            ]);
        });
        $.RULE('inOut', () => {
            $.OR([
                {
                    ALT: () => {
                        $.CONSUME(tokens_1.tokenVocabulary.InKw); // in
                        $.OPTION(() => {
                            $.CONSUME(tokens_1.tokenVocabulary.OutKw); // out
                        });
                    },
                },
                {
                    ALT: () => {
                        $.CONSUME2(tokens_1.tokenVocabulary.OutKw); // out
                    },
                },
            ]);
        });
        $.RULE('argument', () => {
            $.CONSUME(tokens_1.tokenVocabulary.Identifier); // pi_input
            $.OPTION(() => {
                $.SUBRULE($.inOut); // in | out | in out
            });
            $.OPTION2(() => {
                $.CONSUME2(tokens_1.tokenVocabulary.NocopyKw); // nocopy
            });
            $.OR([
                { ALT: () => $.SUBRULE($.dataType) },
                { ALT: () => $.SUBRULE($.objType) },
            ]);
            $.OPTION3(() => {
                $.OR2([
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.DefaultKw) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.Assignment) }, // :=
                ]);
                $.SUBRULE($.value);
            });
        });
        $.RULE('valueInBrackets', () => {
            // more possible in multi dimensional arrays
            $.CONSUME(tokens_1.tokenVocabulary.OpenBracket);
            // $.CONSUME(tokenVocabulary.Identifier);
            $.MANY(() => {
                $.OR([
                    { ALT: () => $.SUBRULE($.value) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.Comma) },
                ]);
            });
            $.CONSUME(tokens_1.tokenVocabulary.ClosingBracket);
        });
        $.RULE('extractValue', () => {
            $.CONSUME(tokens_1.tokenVocabulary.ExtractKw);
            $.CONSUME(tokens_1.tokenVocabulary.OpenBracket);
            $.OR([
                // extract(hour from systimestamp)
                {
                    ALT: () => {
                        $.CONSUME(tokens_1.tokenVocabulary.Identifier);
                        $.CONSUME(tokens_1.tokenVocabulary.FromKw);
                        $.SUBRULE($.value);
                    },
                },
                // xml extract(l_xml, '//xpath', {'namespace'})
                {
                    ALT: () => {
                        $.CONSUME1(tokens_1.tokenVocabulary.Identifier); // xmltype_instance
                        $.CONSUME1(tokens_1.tokenVocabulary.Comma);
                        $.OR2([
                            {
                                ALT: () => $.CONSUME2(tokens_1.tokenVocabulary.Identifier), // variable
                            },
                            {
                                ALT: () => $.CONSUME(tokens_1.tokenVocabulary.StringTk, { LABEL: 'String' }), // xpath
                            },
                        ]);
                        // namespace
                        $.OPTION(() => {
                            $.CONSUME2(tokens_1.tokenVocabulary.Comma);
                            $.OR3([
                                {
                                    ALT: () => $.CONSUME3(tokens_1.tokenVocabulary.Identifier), // variable
                                },
                                {
                                    ALT: () => $.CONSUME1(tokens_1.tokenVocabulary.StringTk, { LABEL: 'String' }), // namespace
                                },
                            ]);
                        });
                    },
                },
            ]);
            $.CONSUME(tokens_1.tokenVocabulary.ClosingBracket);
        });
        // needed to be written out because of the prior rule and extract is its own KW
        $.RULE('xmlExtract', () => {
            // $.CONSUME(tokenVocabulary.Identifier);
            $.CONSUME(tokens_1.tokenVocabulary.Dot);
            $.CONSUME(tokens_1.tokenVocabulary.ExtractKw);
            $.CONSUME(tokens_1.tokenVocabulary.OpenBracket);
            $.OR2([
                {
                    ALT: () => $.CONSUME2(tokens_1.tokenVocabulary.Identifier), // variable
                },
                {
                    ALT: () => $.CONSUME(tokens_1.tokenVocabulary.StringTk, { LABEL: 'String' }), // xpath
                },
            ]);
            // namespace
            $.OPTION(() => {
                $.CONSUME2(tokens_1.tokenVocabulary.Comma);
                $.OR3([
                    {
                        ALT: () => $.CONSUME3(tokens_1.tokenVocabulary.Identifier), // variable
                    },
                    {
                        ALT: () => $.CONSUME1(tokens_1.tokenVocabulary.StringTk, { LABEL: 'String' }), // namespace
                    },
                ]);
            });
            $.CONSUME(tokens_1.tokenVocabulary.ClosingBracket);
        });
        $.RULE('castValue', () => {
            $.CONSUME(tokens_1.tokenVocabulary.CastKw);
            $.CONSUME(tokens_1.tokenVocabulary.OpenBracket);
            $.CONSUME(tokens_1.tokenVocabulary.Identifier);
            $.CONSUME(tokens_1.tokenVocabulary.AsKw);
            $.SUBRULE($.dataType);
            $.CONSUME(tokens_1.tokenVocabulary.ClosingBracket);
        });
        $.RULE('identWithCondDotsExtension', () => {
            $.CONSUME(tokens_1.tokenVocabulary.Dot);
            $.CONSUME(tokens_1.tokenVocabulary.KwIdentifier);
            $.OPTION(() => {
                $.SUBRULE($.valueInBrackets);
            });
        });
        $.RULE('identWithCondDots', () => {
            $.CONSUME(tokens_1.tokenVocabulary.Identifier);
            $.OPTION(() => {
                $.OR([
                    { ALT: () => $.SUBRULE($.xmlExtract) },
                    {
                        ALT: () => {
                            $.OPTION1(() => $.SUBRULE($.valueInBrackets));
                            $.OPTION2(() => {
                                $.MANY(() => {
                                    $.SUBRULE($.identWithCondDotsExtension);
                                });
                            });
                        },
                    },
                ]);
            });
        });
        $.RULE('sqlObjectAll', () => {
            $.CONSUME(tokens_1.tokenVocabulary.Identifier);
            $.CONSUME(tokens_1.tokenVocabulary.Dot);
            $.CONSUME(tokens_1.tokenVocabulary.Asterisk);
        });
        $.RULE('value', (modeSql = false) => {
            $.AT_LEAST_ONE(() => {
                $.OR([
                    { GATE: () => modeSql, ALT: () => $.SUBRULE($.sqlObjectAll) },
                    ...($.XvalueOr ??
                        ($.XvalueOr = [
                            { ALT: () => $.SUBRULE($.extractValue) },
                            { ALT: () => $.SUBRULE($.castValue) },
                            {
                                ALT: () => $.CONSUME(tokens_1.tokenVocabulary.ValueSeperator, {
                                    LABEL: 'Operator',
                                }),
                            },
                            {
                                ALT: () => $.CONSUME(tokens_1.tokenVocabulary.StringTk, { LABEL: 'String' }),
                            },
                            // {
                            //   ALT: () =>
                            //     $.CONSUME(tokenVocabulary.AlternateQuotingMechanism, {
                            //       LABEL: 'String',
                            //     }),
                            // },
                            { ALT: () => $.SUBRULE($.number, { LABEL: 'Number' }) },
                            // {
                            //   ALT: () => $.SUBRULE($.functionCall),
                            // },
                            { ALT: () => $.SUBRULE($.valueInBrackets) },
                            { ALT: () => $.SUBRULE($.sqlCaseStatement) },
                            { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.Null) },
                            { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.ValueKeyword) },
                            { ALT: () => $.SUBRULE($.jsonDtypeValue) },
                            { ALT: () => $.SUBRULE($.identWithCondDots) },
                            // { ALT: () => $.CONSUME(tokenVocabulary.Identifier) },
                            // { ALT: () => $.CONSUME(tokenVocabulary.OpenBracket) },
                            // { ALT: () => $.CONSUME(tokenVocabulary.ClosingBracket) },
                            { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.Arrow) },
                            // { ALT: () => $.CONSUME(tokenVocabulary.FromKw) }, // extract(hour from sysdate)
                            //  { ALT: () => $.CONSUME(tokenVocabulary.Dot) },
                            { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.Percent) },
                            // excluded from 'valueWithoutOperators'
                            { ALT: () => $.SUBRULE($.relationalOperators) },
                            { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.AndOr) },
                            { ALT: () => $.SUBRULE($.nullCheck) },
                            { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.NotKw) },
                            { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.MemberOfKw) },
                            { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.ReplaceKw) },
                            // excluded from 'valueWithoutOperators'
                            {
                                ALT: () => $.CONSUME(tokens_1.tokenVocabulary.InKw), // bool := 1 in (1, 2)
                            },
                        ])),
                ]);
            });
            // $.AT_LEAST_ONE_SEP({
            //   SEP: tokenVocabulary.ValueSeperator,
            //   DEF: () => {
            //     $.OR([
            //       { ALT: () => $.CONSUME(tokenVocabulary.StringTk) },
            //       { ALT: () => $.SUBRULE($.number) },
            //       {
            //         ALT: () => $.SUBRULE($.functionCall),
            //       },
            //       { ALT: () => $.SUBRULE($.valueInBrackets) },
            //       { ALT: () => $.SUBRULE($.sqlCaseStatement) },
            //       { ALT: () => $.CONSUME(tokenVocabulary.Null) },
            //       { ALT: () => $.CONSUME(tokenVocabulary.ValueKeyword) },
            //       { ALT: () => $.SUBRULE($.jsonDtypeValue) },
            //     ]);
            //   },
            // });
        });
        // no 'in' and no relational operators
        $.RULE('valueWithoutOperators', () => {
            $.AT_LEAST_ONE(() => {
                $.OR($.XvalueOrWithoutIn ??
                    ($.XvalueOrWithoutIn = [
                        { ALT: () => $.SUBRULE($.extractValue) },
                        { ALT: () => $.SUBRULE($.xmlExtract) },
                        { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.ValueSeperator) },
                        { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.StringTk) },
                        { ALT: () => $.SUBRULE($.number) },
                        // {
                        //   ALT: () => $.SUBRULE($.functionCall),
                        // },
                        { ALT: () => $.SUBRULE($.valueInBrackets) },
                        { ALT: () => $.SUBRULE($.sqlCaseStatement) },
                        { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.Null) },
                        { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.ValueKeyword) },
                        { ALT: () => $.SUBRULE($.jsonDtypeValue) },
                        { ALT: () => $.SUBRULE($.identWithCondDots) },
                        // { ALT: () => $.CONSUME(tokenVocabulary.Identifier) },
                        // { ALT: () => $.CONSUME(tokenVocabulary.OpenBracket) },
                        // { ALT: () => $.CONSUME(tokenVocabulary.ClosingBracket) },
                        { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.Arrow) },
                        { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.AsKw) },
                        { ALT: () => $.SUBRULE($.dataType) },
                        // { ALT: () => $.CONSUME(tokenVocabulary.Dot) },
                        { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.Percent) },
                        //{ ALT: () => $.CONSUME(tokenVocabulary.AndOr) }, // chain bool vals
                        //{ ALT: () => $.SUBRULE($.nullCheck) }, // bool x is (not) null
                        //{ ALT: () => $.CONSUME(tokenVocabulary.NotKw) },
                        //{ ALT: () => $.CONSUME(tokenVocabulary.MemberOfKw) },
                        { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.ReplaceKw) }, // keyword and also function
                    ]));
            });
        });
        $.RULE('argumentList', () => {
            $.CONSUME(tokens_1.tokenVocabulary.OpenBracket); // (
            $.MANY_SEP({
                SEP: tokens_1.tokenVocabulary.Comma,
                DEF: () => {
                    $.SUBRULE($.argument);
                },
            });
            $.CONSUME(tokens_1.tokenVocabulary.ClosingBracket); // )
        });
        $.RULE('funcSpec', () => {
            $.CONSUME(tokens_1.tokenVocabulary.FunctionKw); // function
            $.CONSUME(tokens_1.tokenVocabulary.Identifier, { LABEL: 'function_name' }); // fnc_name
            $.OPTION(() => {
                $.SUBRULE($.argumentList); // (pi_vc in varchar2, pi_dat in date)
            });
            $.CONSUME(tokens_1.tokenVocabulary.ReturnKw);
            $.OR([
                { ALT: () => $.SUBRULE($.dataType) },
                { ALT: () => $.SUBRULE($.objType) }, // rowtype, pkt type ...
            ]);
            $.OPTION2(() => {
                $.CONSUME(tokens_1.tokenVocabulary.DeterministicKw); // deterministic
            });
            $.OPTION3(() => {
                $.CONSUME(tokens_1.tokenVocabulary.ResultCacheKw); // result_cache
            });
            $.OPTION4(() => {
                $.CONSUME(tokens_1.tokenVocabulary.PipelinedKw); // pipelined
            });
        });
        $.RULE('procSpec', () => {
            $.CONSUME(tokens_1.tokenVocabulary.ProcedureKw); // procedure
            $.CONSUME(tokens_1.tokenVocabulary.Identifier, { LABEL: 'procedure_name' }); // prc_name
            $.OPTION(() => {
                $.SUBRULE($.argumentList); // (pi_vc in varchar2, pi_dat in date)
            });
        });
        $.RULE('dataType', () => {
            $.OR($.XdataTypeOr ??
                ($.XdataTypeOr = [
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.DtypeNumber) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.DtypeDate) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.DtypeTimestamp) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.DtypeTimestampWTZ) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.DtypeBoolean) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.DtypeVarchar2) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.DtypePlsIteger) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.DtypeRaw) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.JsonDtypes) },
                ]));
        });
        // TODO: remove and replace with mathExpression
        $.RULE('numberValue', () => {
            $.OR([
                { ALT: () => $.SUBRULE($.number) },
                { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.Identifier) },
            ]);
            // $.OR({
            //   DEF: [
            //     { ALT: () => $.SUBRULE($.number) },
            //     { ALT: () => $.SUBRULE($.functionCall), IGNORE_AMBIGUITIES: true },
            //     { ALT: () => $.CONSUME(tokenVocabulary.Identifier) },
            //   ],
            // });
        });
        $.RULE('numberDeclaration', () => {
            $.CONSUME(tokens_1.tokenVocabulary.DtypeNumber);
            // (3,2)
            $.OPTION1(() => {
                $.CONSUME(tokens_1.tokenVocabulary.OpenBracket);
                $.CONSUME(tokens_1.tokenVocabulary.Integer);
                $.OPTION2(() => {
                    $.CONSUME(tokens_1.tokenVocabulary.Comma);
                    $.CONSUME2(tokens_1.tokenVocabulary.Integer);
                });
                $.CONSUME(tokens_1.tokenVocabulary.ClosingBracket);
            });
        });
        $.RULE('stringDeclaration', () => {
            $.CONSUME(tokens_1.tokenVocabulary.DtypeVarchar2); // varchar2
            $.OPTION(() => {
                $.CONSUME(tokens_1.tokenVocabulary.OpenBracket); // (
                $.CONSUME(tokens_1.tokenVocabulary.Integer); // 32
                $.OPTION2(() => {
                    $.CONSUME(tokens_1.tokenVocabulary.Char); // char
                });
                $.CONSUME(tokens_1.tokenVocabulary.ClosingBracket); // )
            });
        });
        $.RULE('stringExpression', () => {
            $.AT_LEAST_ONE_SEP({
                SEP: tokens_1.tokenVocabulary.Concat,
                DEF: () => {
                    $.OR([
                        { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.StringTk) },
                        { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.CompilationFlag) },
                        { ALT: () => $.SUBRULE($.functionCall) },
                    ]);
                },
            });
        });
        $.RULE('timestampDeclaration', () => {
            $.OR([
                {
                    ALT: () => {
                        $.CONSUME(tokens_1.tokenVocabulary.DtypeTimestamp); // timestamp
                        $.OPTION(() => {
                            $.CONSUME(tokens_1.tokenVocabulary.OpenBracket); // (
                            $.CONSUME(tokens_1.tokenVocabulary.Integer); // 6
                            $.CONSUME(tokens_1.tokenVocabulary.ClosingBracket); // )
                        });
                    },
                },
                { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.DtypeTimestampWTZ) }, // timestamp with timezone
            ]);
        });
        // $.RULE('variableName', () => {
        //   $.OR([
        //     { ALT: () => $.CONSUME(tokenVocabulary.Identifier) },
        //     { ALT: () => $.CONSUME(tokenVocabulary.AnyValue) },
        //   ]);
        // });
        $.RULE('numberInBrackets', () => {
            $.CONSUME(tokens_1.tokenVocabulary.OpenBracket); // (
            $.OR([
                { ALT: () => $.SUBRULE($.number) },
                { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.Identifier) },
            ]);
            $.CONSUME(tokens_1.tokenVocabulary.ClosingBracket); // )
        });
        $.RULE('assignment', () => {
            $.AT_LEAST_ONE_SEP({
                SEP: tokens_1.tokenVocabulary.Dot,
                DEF: () => {
                    $.CONSUME(tokens_1.tokenVocabulary.Identifier);
                    // associative arrays e. g. l_2d_array(1)(2)
                    $.OPTION(() => {
                        $.MANY(() => {
                            $.SUBRULE($.valueInBrackets);
                        });
                    });
                },
            });
            $.CONSUME(tokens_1.tokenVocabulary.Assignment);
            $.SUBRULE2($.value);
            $.SUBRULE($.semicolon);
        });
        $.RULE('number', () => {
            $.OPTION(() => {
                $.CONSUME(tokens_1.tokenVocabulary.Minus);
            });
            $.OR([
                { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.Integer) },
                { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.Float) },
            ]);
        });
        $.RULE('forallStatement', () => {
            $.CONSUME(tokens_1.tokenVocabulary.ForallKw);
            $.CONSUME(tokens_1.tokenVocabulary.Identifier); // l_var
            $.CONSUME(tokens_1.tokenVocabulary.InKw); // in
            $.SUBRULE($.value); // l_num1
            $.CONSUME(tokens_1.tokenVocabulary.DoubleDot); // ..
            $.SUBRULE2($.value); // 3
            $.SUBRULE($.dmlOperation);
            $.SUBRULE($.semicolon); // ;
        });
        $.RULE('dmlStatement', () => {
            $.SUBRULE($.dmlOperation);
            $.SUBRULE($.semicolon); // ;
        });
        $.RULE('dmlOperation', () => {
            $.OR([
                {
                    ALT: () => $.SUBRULE($.insertOperation),
                },
                {
                    ALT: () => $.SUBRULE($.updateOperation),
                },
                {
                    ALT: () => $.SUBRULE($.deleteOperation),
                },
            ]);
        });
        $.RULE('insertOperation', () => {
            $.CONSUME(tokens_1.tokenVocabulary.InsertKw); // insert
            $.CONSUME(tokens_1.tokenVocabulary.IntoKw); // into
            $.SUBRULE($.dottedIdentifier); // table_name
            $.OR([
                {
                    ALT: () => {
                        $.CONSUME(tokens_1.tokenVocabulary.OpenBracket); // (
                        $.MANY_SEP({
                            SEP: tokens_1.tokenVocabulary.Comma,
                            DEF: () => {
                                $.CONSUME2(tokens_1.tokenVocabulary.Identifier);
                            },
                        });
                        $.CONSUME(tokens_1.tokenVocabulary.ClosingBracket); // )
                        $.OR2([
                            {
                                ALT: () => {
                                    $.CONSUME(tokens_1.tokenVocabulary.ValuesKw); // values
                                    $.CONSUME2(tokens_1.tokenVocabulary.OpenBracket); // (
                                    $.MANY_SEP2({
                                        SEP: tokens_1.tokenVocabulary.Comma,
                                        DEF: () => {
                                            $.SUBRULE($.value);
                                        },
                                    });
                                    $.CONSUME2(tokens_1.tokenVocabulary.ClosingBracket); // )
                                },
                            },
                            {
                                ALT: () => {
                                    $.SUBRULE($.query);
                                },
                            },
                        ]);
                    },
                },
                {
                    ALT: () => {
                        $.CONSUME2(tokens_1.tokenVocabulary.ValuesKw); // values
                        $.CONSUME3(tokens_1.tokenVocabulary.Identifier); // l_rowtype
                    },
                },
            ]);
            $.OPTION(() => {
                $.CONSUME(tokens_1.tokenVocabulary.ReturningKw); // returning
                $.CONSUME4(tokens_1.tokenVocabulary.Identifier); // identity_col
                $.CONSUME2(tokens_1.tokenVocabulary.IntoKw); // into
                $.CONSUME5(tokens_1.tokenVocabulary.Identifier); // variable
            });
        });
        $.RULE('updateOperation', () => {
            $.CONSUME(tokens_1.tokenVocabulary.UpdateKw); // update
            $.SUBRULE($.dottedIdentifier); // table_name
            $.CONSUME(tokens_1.tokenVocabulary.SetKw); // set
            $.OR([
                {
                    ALT: () => {
                        $.AT_LEAST_ONE_SEP({
                            SEP: tokens_1.tokenVocabulary.Comma,
                            DEF: () => {
                                $.CONSUME2(tokens_1.tokenVocabulary.Identifier); // col_x
                                $.CONSUME(tokens_1.tokenVocabulary.Equals); // =
                                $.SUBRULE($.value); // 4
                            },
                        });
                    },
                },
                {
                    ALT: () => {
                        $.CONSUME(tokens_1.tokenVocabulary.RowKw); // row
                        $.CONSUME1(tokens_1.tokenVocabulary.Equals); // =
                        $.CONSUME(tokens_1.tokenVocabulary.Identifier); // l_rowtype
                    },
                },
            ]);
            $.OPTION(() => {
                // where 1 = 1 ...
                $.SUBRULE($.whereClause);
            });
        });
        $.RULE('deleteOperation', () => {
            $.CONSUME(tokens_1.tokenVocabulary.DeleteKw); // delte
            $.OPTION(() => {
                $.CONSUME(tokens_1.tokenVocabulary.FromKw); // from
            });
            $.SUBRULE($.dottedIdentifier); // table_name
            $.OPTION1(() => {
                // where 1 = 1 ...
                $.SUBRULE($.whereClause);
            });
        });
        $.RULE('transactionStatement', () => {
            $.OR([
                { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.StartTransactionKw) },
                { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.CommitKw) },
                { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.RollbackKw) }, // rollback
            ]);
            $.SUBRULE($.semicolon); // ;
        });
        // function or procedure call acutally or schema.pkg.constant
        // or some array velues like myval.arr(2)
        $.RULE('functionCall', () => {
            $.OR([
                { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.Identifier) },
                { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.ReplaceKw) }, // replace is keyword and function name
            ]);
            $.MANY(() => {
                // .subname
                $.CONSUME(tokens_1.tokenVocabulary.Dot);
                $.OR2([
                    {
                        ALT: () => {
                            $.CONSUME2(tokens_1.tokenVocabulary.Identifier);
                            $.OPTION(() => {
                                // (3) for arrays
                                $.SUBRULE($.numberInBrackets);
                            });
                        },
                    },
                    // TODO: Delete this?!
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.DeleteKw) }, // delete on an table array
                ]);
            });
            $.OPTION5(() => {
                $.CONSUME(tokens_1.tokenVocabulary.OpenBracket); // (
                // function without params can be called like fct()
                $.OPTION6(() => {
                    $.MANY_SEP({
                        SEP: tokens_1.tokenVocabulary.Comma,
                        DEF: () => {
                            $.OPTION7(() => {
                                $.CONSUME4(tokens_1.tokenVocabulary.Identifier); // pi_param
                                $.CONSUME(tokens_1.tokenVocabulary.Arrow); // =>
                            });
                            $.SUBRULE($.value); // 6
                        },
                    });
                });
                // optional because will be taken care of in value parser
                //$.OPTION8(() => {
                $.CONSUME(tokens_1.tokenVocabulary.ClosingBracket); // )
                // });
            });
        });
        $.RULE('functionCallSemicolon', () => {
            $.SUBRULE($.functionCall);
            $.SUBRULE($.semicolon);
        });
        $.RULE('whereClause', () => {
            // where 1 = 1 and col2 = 5
            $.CONSUME(tokens_1.tokenVocabulary.WhereKw);
            $.SUBRULE($.chainedConditionsWhere);
        });
        $.RULE('commaSepIdent', () => {
            $.AT_LEAST_ONE_SEP({
                SEP: tokens_1.tokenVocabulary.Comma,
                DEF: () => {
                    $.OPTION(() => {
                        $.CONSUME(tokens_1.tokenVocabulary.Identifier);
                        $.CONSUME(tokens_1.tokenVocabulary.Dot);
                    });
                    $.CONSUME1(tokens_1.tokenVocabulary.Identifier);
                },
            });
        });
        $.RULE('groupClause', () => {
            $.CONSUME(tokens_1.tokenVocabulary.GroupByKw);
            $.OR([
                { ALT: () => $.SUBRULE($.commaSepIdent) },
                {
                    ALT: () => {
                        $.CONSUME(tokens_1.tokenVocabulary.OpenBracket); // (
                        $.SUBRULE1($.commaSepIdent);
                        $.CONSUME(tokens_1.tokenVocabulary.ClosingBracket); // )
                    },
                },
            ]);
            $.OPTION1(() => {
                $.CONSUME(tokens_1.tokenVocabulary.HavingKw);
                $.SUBRULE($.chainedConditions);
            });
        });
        $.RULE('orderClause', () => {
            $.CONSUME(tokens_1.tokenVocabulary.OrderKw);
            $.CONSUME(tokens_1.tokenVocabulary.ByKw);
            $.AT_LEAST_ONE_SEP({
                SEP: tokens_1.tokenVocabulary.Comma,
                DEF: () => {
                    $.SUBRULE($.value); // value to support fc + alias.col
                    $.OPTION(() => {
                        $.CONSUME(tokens_1.tokenVocabulary.AscDescKw);
                    });
                    $.OPTION1(() => {
                        $.CONSUME(tokens_1.tokenVocabulary.NullsFirstLastKw);
                    });
                },
            });
        });
        $.RULE('querySource', () => {
            $.OR([
                { ALT: () => $.SUBRULE($.dottedIdentifier) },
                {
                    // array to table
                    ALT: () => {
                        $.CONSUME(tokens_1.tokenVocabulary.TableKw);
                        $.CONSUME(tokens_1.tokenVocabulary.OpenBracket);
                        $.SUBRULE($.value);
                        $.CONSUME(tokens_1.tokenVocabulary.ClosingBracket);
                    },
                },
            ]);
            $.OPTION1(() => {
                $.CONSUME3(tokens_1.tokenVocabulary.Identifier); // table alias
            });
        });
        $.RULE('tableJoin', () => {
            $.MANY(() => {
                $.OPTION(() => {
                    $.CONSUME(tokens_1.tokenVocabulary.JoinDirection); // left / right / inner / outer
                });
                $.CONSUME(tokens_1.tokenVocabulary.JoinKw);
                $.SUBRULE($.querySource); // table / object
                $.OPTION1(() => {
                    $.CONSUME(tokens_1.tokenVocabulary.OnKw);
                    $.SUBRULE($.chainedConditions);
                });
            });
        });
        $.RULE('dottedIdentifier', () => {
            $.AT_LEAST_ONE_SEP({
                SEP: tokens_1.tokenVocabulary.Dot,
                DEF: () => {
                    $.CONSUME(tokens_1.tokenVocabulary.Identifier);
                },
            });
        });
        $.RULE('withClause', () => {
            $.CONSUME(tokens_1.tokenVocabulary.WithKw);
            $.CONSUME(tokens_1.tokenVocabulary.Identifier);
            $.CONSUME(tokens_1.tokenVocabulary.AsKw);
            $.SUBRULE($.queryInBrackets);
            $.OPTION(() => {
                $.CONSUME(tokens_1.tokenVocabulary.Comma);
                $.AT_LEAST_ONE_SEP({
                    SEP: tokens_1.tokenVocabulary.Comma,
                    DEF: () => {
                        $.CONSUME2(tokens_1.tokenVocabulary.Identifier);
                        $.CONSUME2(tokens_1.tokenVocabulary.AsKw);
                        $.SUBRULE1($.queryInBrackets);
                    },
                });
            });
        });
        $.RULE('queryOverClause', () => {
            $.CONSUME(tokens_1.tokenVocabulary.OverKw);
            $.CONSUME(tokens_1.tokenVocabulary.OpenBracket);
            // partition by col1, col2
            $.OPTION(() => {
                $.CONSUME(tokens_1.tokenVocabulary.PartitionByKw);
                $.AT_LEAST_ONE_SEP({
                    // col1, col2, fct1(3)
                    SEP: tokens_1.tokenVocabulary.Comma,
                    DEF: () => $.SUBRULE($.dottedIdentifier),
                });
            });
            $.OPTION1(() => {
                $.SUBRULE($.orderClause);
            });
            $.CONSUME(tokens_1.tokenVocabulary.ClosingBracket);
        });
        $.RULE('queryWithinGroupClause', () => {
            $.CONSUME(tokens_1.tokenVocabulary.WithinGroupKw);
            $.CONSUME(tokens_1.tokenVocabulary.OpenBracket);
            $.SUBRULE($.orderClause);
            $.CONSUME(tokens_1.tokenVocabulary.ClosingBracket);
        });
        $.RULE('queryColumns', () => {
            $.AT_LEAST_ONE_SEP({
                // col1, col2, fct1(3)
                SEP: tokens_1.tokenVocabulary.Comma,
                DEF: () => {
                    // also includes just >> * <<
                    $.SUBRULE($.value, { ARGS: [true] }); // direct value / function call / variable
                    $.OPTION(() => {
                        $.SUBRULE($.queryWithinGroupClause);
                    });
                    $.OPTION1(() => {
                        $.SUBRULE($.queryOverClause);
                    });
                    $.OPTION4(() => {
                        $.SUBRULE($.queryColumnAlias);
                    });
                },
            });
        });
        $.RULE('queryColumnAlias', () => {
            $.OPTION(() => {
                $.CONSUME(tokens_1.tokenVocabulary.AsKw);
            });
            debugger;
            $.CONSUME(tokens_1.tokenVocabulary.KwIdentifier); // alias
        });
        $.RULE('sqlJsonTableColumn', () => {
            $.CONSUME(tokens_1.tokenVocabulary.Identifier);
            $.OPTION(() => {
                $.SUBRULE($.variableSpec);
            });
            $.OPTION1(() => {
                $.CONSUME(tokens_1.tokenVocabulary.FormatJsonKw);
            });
            $.CONSUME(tokens_1.tokenVocabulary.PathKw);
            $.CONSUME(tokens_1.tokenVocabulary.StringTk);
        });
        $.RULE('sqlJsonTable', () => {
            $.CONSUME(tokens_1.tokenVocabulary.JsonTableKw);
            $.CONSUME(tokens_1.tokenVocabulary.OpenBracket);
            $.SUBRULE($.value);
            $.CONSUME(tokens_1.tokenVocabulary.Comma);
            $.CONSUME(tokens_1.tokenVocabulary.StringTk);
            $.CONSUME(tokens_1.tokenVocabulary.ColumnsKw);
            $.OR([
                {
                    ALT: () => {
                        $.AT_LEAST_ONE_SEP({
                            SEP: tokens_1.tokenVocabulary.Comma,
                            DEF: () => {
                                $.SUBRULE($.sqlJsonTableColumn);
                            },
                        });
                    },
                },
                {
                    ALT: () => {
                        $.CONSUME1(tokens_1.tokenVocabulary.OpenBracket);
                        $.AT_LEAST_ONE_SEP1({
                            SEP: tokens_1.tokenVocabulary.Comma,
                            DEF: () => {
                                $.SUBRULE1($.sqlJsonTableColumn);
                            },
                        });
                        $.CONSUME1(tokens_1.tokenVocabulary.ClosingBracket);
                    },
                },
            ]);
            $.CONSUME(tokens_1.tokenVocabulary.ClosingBracket);
            $.OPTION1(() => {
                $.CONSUME3(tokens_1.tokenVocabulary.Identifier); // alias
            });
        });
        $.RULE('sqlFromClause', () => {
            $.CONSUME(tokens_1.tokenVocabulary.FromKw); // from
            $.AT_LEAST_ONE_SEP({
                // table 1, table 2
                SEP: tokens_1.tokenVocabulary.Comma,
                DEF: () => {
                    $.OR([
                        { ALT: () => $.SUBRULE($.querySource) },
                        {
                            ALT: () => {
                                $.SUBRULE($.queryInBrackets);
                                $.OPTION1(() => {
                                    $.CONSUME3(tokens_1.tokenVocabulary.Identifier); // alias
                                });
                            },
                        },
                        {
                            ALT: () => $.SUBRULE($.sqlJsonTable),
                        },
                    ]);
                },
            });
        });
        $.RULE('offsetQueryClause', () => {
            $.CONSUME(tokens_1.tokenVocabulary.OffsetKw);
            $.OR([
                { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.Integer) },
                { ALT: () => $.SUBRULE($.dottedIdentifier) },
            ]);
            $.CONSUME(tokens_1.tokenVocabulary.RowKw);
        });
        $.RULE('fetchQueryClause', () => {
            $.OR([
                { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.FetchFirstLastKw) },
                { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.FetchKw) },
            ]);
            $.OPTION1(() => {
                $.CONSUME(tokens_1.tokenVocabulary.Integer);
                $.OPTION2(() => {
                    $.CONSUME(tokens_1.tokenVocabulary.PercentKw);
                });
            });
            $.CONSUME(tokens_1.tokenVocabulary.RowKw);
            $.OR1([
                { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.WithTiesKw) },
                { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.OnlyKw) },
            ]);
        });
        $.RULE('queryInBrackets', () => {
            $.CONSUME(tokens_1.tokenVocabulary.OpenBracket);
            $.SUBRULE($.query);
            $.CONSUME(tokens_1.tokenVocabulary.ClosingBracket);
        });
        // TODO union, minus, intersect
        // TODO distinct
        $.RULE('query', () => {
            $.OPTION9(() => {
                $.SUBRULE($.withClause);
            });
            $.CONSUME(tokens_1.tokenVocabulary.SelectKw); // select
            $.SUBRULE($.queryColumns);
            $.OPTION1(() => {
                $.CONSUME1(tokens_1.tokenVocabulary.IntoKw); // into
                $.AT_LEAST_ONE_SEP2({
                    // into l_val1, l_val2
                    SEP: tokens_1.tokenVocabulary.Comma,
                    DEF: () => {
                        $.SUBRULE($.dottedIdentifier);
                    },
                });
            });
            $.SUBRULE($.sqlFromClause);
            $.SUBRULE($.tableJoin);
            $.OPTION2(() => {
                $.SUBRULE($.whereClause);
            });
            $.OPTION3(() => {
                $.SUBRULE($.groupClause);
            });
            $.OPTION4(() => {
                $.SUBRULE($.orderClause);
            });
            $.OPTION5(() => {
                $.SUBRULE($.offsetQueryClause);
            });
            $.OPTION6(() => {
                $.SUBRULE($.fetchQueryClause);
            });
        });
        $.RULE('queryStatement', () => {
            $.SUBRULE($.query); // select ...
            $.SUBRULE($.semicolon); // ;
        });
        $.RULE('dynamicSqlStatement', () => {
            $.CONSUME(tokens_1.tokenVocabulary.ExecuteImmediateKw); // execute immediate
            $.SUBRULE($.stringExpression); // l_var or 'create table...'
            $.OPTION(() => {
                $.CONSUME(tokens_1.tokenVocabulary.IntoKw); // into
                $.AT_LEAST_ONE_SEP({
                    SEP: tokens_1.tokenVocabulary.Comma,
                    DEF: () => {
                        $.CONSUME(tokens_1.tokenVocabulary.Identifier); // l_var, l_rec
                    },
                });
            });
            $.OPTION2(() => {
                $.CONSUME(tokens_1.tokenVocabulary.UsingKw); // using
                $.AT_LEAST_ONE_SEP2({
                    SEP: tokens_1.tokenVocabulary.Comma,
                    DEF: () => {
                        $.OPTION3(() => {
                            $.SUBRULE($.inOut); // in | out | in out
                        });
                        $.CONSUME2(tokens_1.tokenVocabulary.Identifier); // l_var, l_var2
                    },
                });
            });
            $.OPTION4(() => {
                $.OR([
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.ReturnKw) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.ReturningKw) }, // returning
                ]);
                $.CONSUME2(tokens_1.tokenVocabulary.IntoKw); // into
                $.AT_LEAST_ONE_SEP3({
                    SEP: tokens_1.tokenVocabulary.Comma,
                    DEF: () => {
                        $.CONSUME3(tokens_1.tokenVocabulary.Identifier); // l_var, l_var2
                    },
                });
            });
            $.SUBRULE($.semicolon);
        });
        $.RULE('exceptionBlock', () => {
            $.CONSUME(tokens_1.tokenVocabulary.ExceptionKw); // exception
            $.MANY(() => {
                $.CONSUME(tokens_1.tokenVocabulary.WhenKw); // when
                $.OR([
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.DefinedException) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.OthersKw) },
                    { ALT: () => $.CONSUME(tokens_1.tokenVocabulary.Identifier) }, // user_defined_exception
                ]);
                $.MANY1(() => {
                    $.CONSUME(tokens_1.tokenVocabulary.OrKw); // or
                    $.OR1([
                        { ALT: () => $.CONSUME1(tokens_1.tokenVocabulary.DefinedException) },
                        { ALT: () => $.CONSUME1(tokens_1.tokenVocabulary.OthersKw) },
                        { ALT: () => $.CONSUME1(tokens_1.tokenVocabulary.Identifier) }, // user_defined_exception
                    ]);
                });
                $.CONSUME(tokens_1.tokenVocabulary.Then); // then
                $.MANY2(() => {
                    $.SUBRULE($.statement);
                });
            });
        });
        $.RULE('whileLoop', () => {
            $.CONSUME(tokens_1.tokenVocabulary.WhileKw); // for
            $.SUBRULE($.chainedConditions);
            $.CONSUME(tokens_1.tokenVocabulary.LoopKw);
            $.MANY(() => {
                $.SUBRULE($.statement);
            });
            $.CONSUME(tokens_1.tokenVocabulary.End);
            $.CONSUME2(tokens_1.tokenVocabulary.LoopKw);
            $.SUBRULE($.semicolon);
        });
        $.RULE('forLoop', () => {
            $.CONSUME(tokens_1.tokenVocabulary.ForKw); // for
            $.CONSUME(tokens_1.tokenVocabulary.Identifier); // l_var
            $.CONSUME(tokens_1.tokenVocabulary.InKw); // in
            $.OR([
                {
                    ALT: () => {
                        $.SUBRULE($.value); // l_num1
                        $.CONSUME(tokens_1.tokenVocabulary.DoubleDot); // ..
                        $.SUBRULE2($.value); // 3
                    },
                },
                {
                    ALT: () => {
                        $.SUBRULE($.queryInBrackets);
                    },
                },
                {
                    ALT: () => {
                        $.CONSUME2(tokens_1.tokenVocabulary.Identifier); // cursor_name
                    },
                },
            ]);
            $.CONSUME(tokens_1.tokenVocabulary.LoopKw);
            $.MANY(() => {
                $.SUBRULE($.statement);
            });
            $.CONSUME(tokens_1.tokenVocabulary.End);
            $.CONSUME2(tokens_1.tokenVocabulary.LoopKw);
            $.SUBRULE($.semicolon);
        });
        $.RULE('exitLoop', () => {
            $.CONSUME(tokens_1.tokenVocabulary.LoopKw);
            $.MANY(() => {
                $.SUBRULE($.statement);
            });
            $.CONSUME(tokens_1.tokenVocabulary.End);
            $.CONSUME2(tokens_1.tokenVocabulary.LoopKw);
            $.SUBRULE($.semicolon);
        });
        $.RULE('createSequenceStatement', () => {
            $.CONSUME(tokens_1.tokenVocabulary.CreateKw); // create
            $.CONSUME(tokens_1.tokenVocabulary.SequenceKw); // sequence
            $.OPTION(() => {
                $.CONSUME(tokens_1.tokenVocabulary.Identifier, { LABEL: 'schema_name' }); // schema_name
                $.CONSUME(tokens_1.tokenVocabulary.Dot); // .
            });
            $.CONSUME1(tokens_1.tokenVocabulary.Identifier, { LABEL: 'sequence_name' }); // sequence_name
            // all the possible options...
            // order of statements does not matter -> many
            // unfortunately because of many the parser does not catch when a option appears multiple times
            $.MANY(() => {
                $.OR([
                    {
                        ALT: () => {
                            $.CONSUME(tokens_1.tokenVocabulary.IncrementByKw); // increment by
                            $.CONSUME(tokens_1.tokenVocabulary.Integer, { LABEL: 'increment_amount' }); // 32
                        },
                    },
                    {
                        ALT: () => {
                            $.CONSUME(tokens_1.tokenVocabulary.StartWithKw); // start with
                            $.CONSUME1(tokens_1.tokenVocabulary.Integer, { LABEL: 'start_with_num' }); // 32
                        },
                    },
                    {
                        ALT: () => {
                            $.OR1([
                                {
                                    ALT: () => {
                                        $.CONSUME(tokens_1.tokenVocabulary.MaxvalueKw); // maxvalue
                                        $.CONSUME3(tokens_1.tokenVocabulary.Integer, {
                                            LABEL: 'maxvalue_num',
                                        });
                                    },
                                },
                                {
                                    ALT: () => $.CONSUME(tokens_1.tokenVocabulary.NoMaxvalueKw), // nomaxvalue
                                },
                            ]);
                        },
                    },
                    {
                        ALT: () => {
                            $.OR2([
                                {
                                    ALT: () => {
                                        $.CONSUME(tokens_1.tokenVocabulary.MinvalueKw); // minvalue
                                        $.CONSUME4(tokens_1.tokenVocabulary.Integer, {
                                            LABEL: 'minvalue_num',
                                        });
                                    },
                                },
                                {
                                    ALT: () => $.CONSUME(tokens_1.tokenVocabulary.NoMinvalueKw), // nominvalue
                                },
                            ]);
                        },
                    },
                    {
                        ALT: () => {
                            $.OR3([
                                {
                                    ALT: () => $.CONSUME(tokens_1.tokenVocabulary.CycleKw), // cycle
                                },
                                {
                                    ALT: () => $.CONSUME(tokens_1.tokenVocabulary.NoCycleKw), // nocycle
                                },
                            ]);
                        },
                    },
                    {
                        ALT: () => {
                            $.OR4([
                                {
                                    ALT: () => {
                                        $.CONSUME(tokens_1.tokenVocabulary.CacheKw); // cache
                                        $.CONSUME5(tokens_1.tokenVocabulary.Integer, { LABEL: 'cache_num' }); // 32
                                    },
                                },
                                {
                                    ALT: () => $.CONSUME(tokens_1.tokenVocabulary.NoCacheKw), // nocache
                                },
                            ]);
                        },
                    },
                    {
                        ALT: () => {
                            $.OR5([
                                {
                                    ALT: () => $.CONSUME(tokens_1.tokenVocabulary.OrderKw), // order
                                },
                                {
                                    ALT: () => $.CONSUME(tokens_1.tokenVocabulary.NoOrderKw), // noorder
                                },
                            ]);
                        },
                    },
                ]);
            });
            $.SUBRULE($.semicolon);
        });
        $.RULE('createViewStatement', () => {
            $.CONSUME(tokens_1.tokenVocabulary.CreateKw); // create
            $.OPTION(() => {
                $.CONSUME(tokens_1.tokenVocabulary.OrKw); // or
                $.CONSUME(tokens_1.tokenVocabulary.ReplaceKw); // replace
            });
            $.OPTION1(() => {
                $.OPTION2(() => {
                    $.CONSUME(tokens_1.tokenVocabulary.NoKw); // no
                });
                $.CONSUME(tokens_1.tokenVocabulary.ForceKw); // force
            });
            $.CONSUME(tokens_1.tokenVocabulary.ViewKw); // view
            $.OPTION3(() => {
                $.CONSUME(tokens_1.tokenVocabulary.Identifier, { LABEL: 'schema_name' }); // schema_name
                $.CONSUME(tokens_1.tokenVocabulary.Dot); // .
            });
            $.CONSUME1(tokens_1.tokenVocabulary.Identifier, { LABEL: 'view_name' }); // view_name
            $.CONSUME(tokens_1.tokenVocabulary.AsKw); // as
            $.SUBRULE($.query); // query
            $.SUBRULE($.semicolon);
        });
        $.RULE('semicolon', () => {
            $.CONSUME(tokens_1.tokenVocabulary.Semicolon, {
                ERR_MSG: 'expteted ";" at the end of the statement',
            });
        });
        // very important to call this after all the rules have been defined.
        // otherwise the parser may not work correctly as it will lack information
        // derived during the self analysis phase.
        this.performSelfAnalysis();
    }
}
exports.default = PlSqlParser;
//# sourceMappingURL=rules.js.map