"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chevrotain_1 = require("chevrotain");
const Identifier_1 = require("./tokens/Identifier");
const tokenBaseCollection = [];
const tokenMap = new Map();
tokenMap.set('Identifier', Identifier_1.default);
function getCategory(lexerCategory) {
    if (!lexerCategory) {
        return undefined;
    }
    if (!tokenMap.has(lexerCategory)) {
        throw new Error(`Token category ${lexerCategory} is not present in TokenMap`);
    }
    else {
        return tokenMap.get(lexerCategory);
    }
}
function getLongerAlt(longerAlt) {
    if (longerAlt) {
        if (!tokenMap.has(longerAlt)) {
            throw new Error(`Longer alt token ${longerAlt} is not present in TokenMap`);
        }
        else {
            return Object.assign({}, tokenMap.get(longerAlt));
        }
    }
    else {
        return undefined;
    }
}
function convertToken(tbt) {
    tokenBaseCollection.push(tbt);
    const tokenArgs = {
        name: tbt.name,
        pattern: tbt.lexerNa === true ? chevrotain_1.Lexer.NA : tbt.pattern,
    };
    if (tbt.longerAlt) {
        tokenArgs.longer_alt = getLongerAlt(tbt.longerAlt);
    }
    if (tbt.lexerCategory) {
        tokenArgs.categories = getCategory(tbt.lexerCategory);
    }
    const token = (0, chevrotain_1.createToken)(tokenArgs);
    console.log(token);
    tokenMap.set(token.name, token);
    return token;
}
exports.default = convertToken;
//# sourceMappingURL=convertToken.js.map