"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chevrotain_1 = require("chevrotain");
const Identifier_1 = require("./Identifier");
const exception_1 = require("./exception");
const DtypeNumber = (0, chevrotain_1.createToken)({
    name: 'DtypeNumber',
    pattern: /number/i,
    longer_alt: Identifier_1.default,
});
const DtypePlsIteger = (0, chevrotain_1.createToken)({
    name: 'DtypePlsIteger',
    pattern: /pls_integer/i,
    longer_alt: Identifier_1.default,
});
const DtypeVarchar2 = (0, chevrotain_1.createToken)({
    name: 'DtypeVarchar2',
    pattern: /varchar2/i,
    longer_alt: Identifier_1.default,
});
const DtypeBoolean = (0, chevrotain_1.createToken)({
    name: 'DtypeBoolean',
    pattern: /boolean/i,
    longer_alt: Identifier_1.default,
});
const DtypeDate = (0, chevrotain_1.createToken)({
    name: 'DtypeDate',
    pattern: /date/i,
    longer_alt: Identifier_1.default,
});
const DtypeTimestampWTZ = (0, chevrotain_1.createToken)({
    name: 'DtypeTimestampWTZ',
    pattern: /timestamp with time zone/i,
    longer_alt: Identifier_1.default,
});
const DtypeTimestamp = (0, chevrotain_1.createToken)({
    name: 'DtypeTimestamp',
    pattern: /timestamp/i,
    longer_alt: [DtypeTimestampWTZ, Identifier_1.default],
});
const DtypeRaw = (0, chevrotain_1.createToken)({
    name: 'DtypeRaw',
    pattern: /raw/i,
    longer_alt: Identifier_1.default,
});
const Char = (0, chevrotain_1.createToken)({
    name: 'Char',
    pattern: /char/i,
    longer_alt: Identifier_1.default,
});
const JsonDtypes = (0, chevrotain_1.createToken)({
    name: 'JsonDtypes',
    pattern: chevrotain_1.Lexer.NA,
});
const JsonObjectT = (0, chevrotain_1.createToken)({
    name: 'JsonObjectT',
    pattern: /json_object_t/i,
    longer_alt: Identifier_1.default,
    categories: JsonDtypes,
});
const JsonArrayT = (0, chevrotain_1.createToken)({
    name: 'JsonArrayT',
    pattern: /json_array_t/i,
    longer_alt: Identifier_1.default,
    categories: JsonDtypes,
});
const JsonElementT = (0, chevrotain_1.createToken)({
    name: 'JsonElementT',
    pattern: /json_element_t/i,
    longer_alt: Identifier_1.default,
    categories: JsonDtypes,
});
const JsonScalarT = (0, chevrotain_1.createToken)({
    name: 'JsonScalarT',
    pattern: /json_scalar_t/i,
    longer_alt: Identifier_1.default,
    categories: JsonDtypes,
});
const Rowtype = (0, chevrotain_1.createToken)({
    name: 'Rowtype',
    pattern: /rowtype/i,
    longer_alt: [
        exception_1.default.find((t) => t.name === 'RowtypeMismatchKw'),
        Identifier_1.default,
    ],
});
const Type = (0, chevrotain_1.createToken)({
    name: 'Type',
    pattern: /type/i,
    longer_alt: Identifier_1.default,
});
const OffsetKw = (0, chevrotain_1.createToken)({
    name: 'OffsetKw',
    pattern: /offset/i,
    longer_alt: Identifier_1.default,
});
const OfKw = (0, chevrotain_1.createToken)({
    name: 'OfKw',
    pattern: /of/i,
    longer_alt: [OffsetKw, Identifier_1.default],
});
const MemberOfKw = (0, chevrotain_1.createToken)({
    name: 'MemberOfKw',
    pattern: /member\s+of/i,
    longer_alt: Identifier_1.default,
});
const IndexKw = (0, chevrotain_1.createToken)({
    name: 'IndexKw',
    pattern: /index/i,
    longer_alt: Identifier_1.default,
});
const ByKw = (0, chevrotain_1.createToken)({
    name: 'ByKw',
    pattern: /by/i,
    longer_alt: Identifier_1.default,
});
const RecordKw = (0, chevrotain_1.createToken)({
    name: 'RecordKw',
    pattern: /record/i,
    longer_alt: Identifier_1.default,
});
const ConstantKw = (0, chevrotain_1.createToken)({
    name: 'ConstantKw',
    pattern: /constant/i,
    longer_alt: Identifier_1.default,
});
exports.default = [
    DtypeNumber,
    DtypePlsIteger,
    DtypeVarchar2,
    DtypeBoolean,
    DtypeDate,
    DtypeTimestampWTZ,
    DtypeTimestamp,
    DtypeRaw,
    Char,
    JsonDtypes,
    JsonObjectT,
    JsonArrayT,
    JsonElementT,
    JsonScalarT,
    Rowtype,
    Type,
    OffsetKw,
    OfKw,
    MemberOfKw,
    IndexKw,
    ByKw,
    RecordKw,
    ConstantKw,
];
//# sourceMappingURL=dataTypes.js.map