"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.lex = exports.tokenVocabulary = void 0;
const chevrotain_1 = require("chevrotain");
const dataTypes_1 = require("./dataTypes");
const dynSql_1 = require("./dynSql");
const exception_1 = require("./exception");
const Identifier_1 = require("./Identifier");
const ifStatement_1 = require("./ifStatement");
const loops_1 = require("./loops");
const pragma_1 = require("./pragma");
const relationalOperators_1 = require("./relationalOperators");
const sql_1 = require("./sql");
const subprograms_1 = require("./subprograms");
const symbols_1 = require("./symbols");
const transaction_1 = require("./transaction");
const values_1 = require("./values");
const strings_1 = require("./strings");
const objectTokens_1 = require("./objectTokens");
const specialIdentifiers_1 = require("./specialIdentifiers");
/* ===== Keywords ===== */
const Declare = (0, chevrotain_1.createToken)({
    name: 'Declare',
    pattern: /declare/i,
    // longer_alt: Identifier
});
const Begin = (0, chevrotain_1.createToken)({
    name: 'Begin',
    pattern: /begin/i,
    longer_alt: Identifier_1.default,
});
const End = (0, chevrotain_1.createToken)({
    name: 'End',
    pattern: /end/i,
    longer_alt: Identifier_1.default,
});
const CreateKw = (0, chevrotain_1.createToken)({
    name: 'CreateKw',
    pattern: /create/i,
    longer_alt: Identifier_1.default,
});
const ReplaceKw = (0, chevrotain_1.createToken)({
    name: 'ReplaceKw',
    pattern: /replace/i,
    longer_alt: Identifier_1.default,
});
const AsIs = (0, chevrotain_1.createToken)({
    name: 'AsIs',
    pattern: chevrotain_1.Lexer.NA,
});
const AsKw = (0, chevrotain_1.createToken)({
    name: 'AsKw',
    pattern: /as/i,
    longer_alt: Identifier_1.default,
    categories: AsIs,
});
const IsKw = (0, chevrotain_1.createToken)({
    name: 'IsKw',
    pattern: /is/i,
    longer_alt: Identifier_1.default,
    categories: AsIs,
});
const ExitKw = (0, chevrotain_1.createToken)({
    name: 'ExitKw',
    pattern: /exit/i,
    longer_alt: Identifier_1.default,
});
const TableKw = (0, chevrotain_1.createToken)({
    name: 'TableKw',
    pattern: /table/i,
    longer_alt: Identifier_1.default,
});
const CursorKw = (0, chevrotain_1.createToken)({
    name: 'CursorKw',
    pattern: /cursor/i,
    longer_alt: Identifier_1.default,
});
const ExtractKw = (0, chevrotain_1.createToken)({
    name: 'ExtractKw',
    pattern: /extract/i,
    longer_alt: Identifier_1.default,
});
const RowKw = (0, chevrotain_1.createToken)({
    name: 'RowKw',
    pattern: /row[s]?/i,
    longer_alt: [
        Identifier_1.default,
        exception_1.default.find((t) => t.name === 'RowtypeMismatchKw'),
    ],
});
const NoKw = (0, chevrotain_1.createToken)({
    name: 'NoKw',
    pattern: /no/i,
    longer_alt: [
        objectTokens_1.default.find((t) => t.name === 'NoCycleKw'),
        objectTokens_1.default.find((t) => t.name === 'NoMaxvalueKw'),
        objectTokens_1.default.find((t) => t.name === 'NoMinvalueKw'),
        objectTokens_1.default.find((t) => t.name === 'NoCacheKw'),
        objectTokens_1.default.find((t) => t.name === 'NoOrderKw'),
        subprograms_1.default.find((t) => t.name === 'NocopyKw'),
        relationalOperators_1.default.find((t) => t.name === 'NotKw'),
        Identifier_1.default,
    ],
});
const ForceKw = (0, chevrotain_1.createToken)({
    name: 'ForceKw',
    pattern: /force/i,
    longer_alt: Identifier_1.default,
    categories: [specialIdentifiers_1.KwIdentifier],
});
const ForallKw = (0, chevrotain_1.createToken)({
    name: 'ForallKw',
    pattern: /forall/i,
    longer_alt: Identifier_1.default,
});
const FormatJsonKw = (0, chevrotain_1.createToken)({
    name: 'FormatJsonKw',
    pattern: /format\s+json/i,
    longer_alt: Identifier_1.default,
});
const ForKw = (0, chevrotain_1.createToken)({
    name: 'ForKw',
    pattern: /for/i,
    longer_alt: [ForceKw, ForallKw, FormatJsonKw, Identifier_1.default],
});
const CastKw = (0, chevrotain_1.createToken)({
    name: 'CastKw',
    pattern: /cast/i,
    longer_alt: Identifier_1.default,
    categories: [specialIdentifiers_1.KwIdentifier],
});
const Float = (0, chevrotain_1.createToken)({
    name: 'Float',
    pattern: /([0-9]*[.])[0-9]+/,
});
const Integer = (0, chevrotain_1.createToken)({
    name: 'Integer',
    pattern: /0|[1-9]\d*/,
});
const Quote = (0, chevrotain_1.createToken)({
    name: 'Quote',
    pattern: /"/,
    group: chevrotain_1.Lexer.SKIPPED,
});
const WhiteSpace = (0, chevrotain_1.createToken)({
    name: 'WhiteSpace',
    pattern: /[ \t\n\r]+/,
    group: chevrotain_1.Lexer.SKIPPED,
});
const allTokens = [
    WhiteSpace,
    Quote,
    // "keywords" appear before the Identifier
    ...pragma_1.default,
    ...exception_1.default,
    ...ifStatement_1.default,
    Declare,
    Begin,
    End,
    ForceKw,
    ForallKw,
    FormatJsonKw,
    ForKw,
    CastKw,
    ...dataTypes_1.default,
    ...values_1.default,
    ...sql_1.default,
    ...dynSql_1.default,
    ...transaction_1.default,
    ...loops_1.default,
    CreateKw,
    ReplaceKw,
    AsIs,
    AsKw,
    IsKw,
    ExitKw,
    TableKw,
    CursorKw,
    ExtractKw,
    RowKw,
    ...objectTokens_1.default,
    ...subprograms_1.default,
    ...symbols_1.default,
    ...relationalOperators_1.default,
    NoKw,
    // The Identifier must appear after the keywords because all keywords are valid identifiers.
    specialIdentifiers_1.KwIdentifier,
    Identifier_1.default,
    Float,
    Integer,
    ...strings_1.default,
];
const SelectLexer = new chevrotain_1.Lexer(allTokens, { positionTracking: 'full' });
// the vocabulary will be exported and used in the Parser definition.
exports.tokenVocabulary = {};
allTokens.forEach((tokenType) => {
    exports.tokenVocabulary[tokenType.name] = tokenType;
});
function lex(inputText) {
    const lexingResult = SelectLexer.tokenize(inputText);
    if (lexingResult.errors && lexingResult.errors.length > 0) {
        //console.log('errors: ', lexingResult.errors);
        throw Error(`Lexing errors detected: ${JSON.stringify(lexingResult.errors)}`);
    }
    return lexingResult;
}
exports.lex = lex;
//# sourceMappingURL=index.js.map