"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chevrotain_1 = require("chevrotain");
const Identifier_1 = require("./Identifier");
const specialIdentifiers_1 = require("./specialIdentifiers");
const SelectKw = (0, chevrotain_1.createToken)({
    name: 'SelectKw',
    pattern: /select/i,
    longer_alt: Identifier_1.default,
});
const IntoKw = (0, chevrotain_1.createToken)({
    name: 'IntoKw',
    pattern: /into/i,
    longer_alt: Identifier_1.default,
});
const FromKw = (0, chevrotain_1.createToken)({
    name: 'FromKw',
    pattern: /from/i,
    longer_alt: Identifier_1.default,
});
const WhereKw = (0, chevrotain_1.createToken)({
    name: 'WhereKw',
    pattern: /where/i,
    longer_alt: Identifier_1.default,
});
const GroupByKw = (0, chevrotain_1.createToken)({
    name: 'GroupByKw',
    pattern: /group by/i,
    longer_alt: Identifier_1.default,
});
const OrderKw = (0, chevrotain_1.createToken)({
    name: 'OrderKw',
    pattern: /order/i,
    longer_alt: Identifier_1.default,
});
const AscDescKw = (0, chevrotain_1.createToken)({
    name: 'AscDescKw',
    pattern: /asc|desc/i,
    longer_alt: Identifier_1.default,
});
const NullsFirstLastKw = (0, chevrotain_1.createToken)({
    name: 'NullsFirstLastKw',
    pattern: /nulls\s+(first|last)/i,
    longer_alt: Identifier_1.default,
});
const Null = (0, chevrotain_1.createToken)({
    name: 'Null',
    pattern: /null/i,
    longer_alt: NullsFirstLastKw,
});
const AndOr = (0, chevrotain_1.createToken)({
    name: 'AndOr',
    pattern: chevrotain_1.Lexer.NA,
});
const AndKw = (0, chevrotain_1.createToken)({
    name: 'AndKw',
    pattern: /and/i,
    longer_alt: Identifier_1.default,
    categories: AndOr,
});
const OrKw = (0, chevrotain_1.createToken)({
    name: 'OrKw',
    pattern: /or/i,
    longer_alt: Identifier_1.default,
    categories: AndOr,
});
const JoinKw = (0, chevrotain_1.createToken)({
    name: 'JoinKw',
    pattern: /join/i,
    longer_alt: Identifier_1.default,
});
const JoinDirection = (0, chevrotain_1.createToken)({
    name: 'JoinDirection',
    pattern: chevrotain_1.Lexer.NA,
});
const LeftKw = (0, chevrotain_1.createToken)({
    name: 'LeftKw',
    pattern: /left/i,
    longer_alt: Identifier_1.default,
    categories: JoinDirection,
});
const RightKw = (0, chevrotain_1.createToken)({
    name: 'RightKw',
    pattern: /right/i,
    longer_alt: Identifier_1.default,
    categories: JoinDirection,
});
const InnerKw = (0, chevrotain_1.createToken)({
    name: 'InnerKw',
    pattern: /inner/i,
    longer_alt: Identifier_1.default,
    categories: JoinDirection,
});
const OuterKw = (0, chevrotain_1.createToken)({
    name: 'OuterKw',
    pattern: /outer/i,
    longer_alt: Identifier_1.default,
    categories: JoinDirection,
});
const CrossKw = (0, chevrotain_1.createToken)({
    name: 'CrossKw',
    pattern: /cross/i,
    longer_alt: Identifier_1.default,
    categories: JoinDirection,
});
const InsertKw = (0, chevrotain_1.createToken)({
    name: 'InsertKw',
    pattern: /insert/i,
    longer_alt: Identifier_1.default,
});
const ValuesKw = (0, chevrotain_1.createToken)({
    name: 'ValuesKw',
    pattern: /values/i,
    longer_alt: Identifier_1.default,
});
const DeleteKw = (0, chevrotain_1.createToken)({
    name: 'DeleteKw',
    pattern: /delete/i,
    longer_alt: Identifier_1.default,
});
const UpdateKw = (0, chevrotain_1.createToken)({
    name: 'UpdateKw',
    pattern: /update/i,
    longer_alt: Identifier_1.default,
});
const SetKw = (0, chevrotain_1.createToken)({
    name: 'SetKw',
    pattern: /set/i,
    longer_alt: Identifier_1.default,
});
const HavingKw = (0, chevrotain_1.createToken)({
    name: 'HavingKw',
    pattern: /having/i,
    longer_alt: Identifier_1.default,
});
const OverKw = (0, chevrotain_1.createToken)({
    name: 'OverKw',
    pattern: /over/i,
    longer_alt: Identifier_1.default,
});
const PartitionByKw = (0, chevrotain_1.createToken)({
    name: 'PartitionByKw',
    pattern: /partition\s+by/i,
    longer_alt: Identifier_1.default,
});
const WithinGroupKw = (0, chevrotain_1.createToken)({
    name: 'WithinGroupKw',
    pattern: /within\s+group/i,
    longer_alt: Identifier_1.default,
});
// dont want single tokens for first / last as they are also function names
const FetchFirstLastKw = (0, chevrotain_1.createToken)({
    name: 'FetchFirstLastKw',
    pattern: /fetch\s+(first|last|next)/i,
    longer_alt: Identifier_1.default,
});
const FetchKw = (0, chevrotain_1.createToken)({
    name: 'FetchKw',
    pattern: /fetch/i,
    longer_alt: [FetchFirstLastKw, Identifier_1.default],
});
const PercentKw = (0, chevrotain_1.createToken)({
    name: 'PercentKw',
    pattern: /percent/i,
    longer_alt: Identifier_1.default,
});
const OnlyKw = (0, chevrotain_1.createToken)({
    name: 'OnlyKw',
    pattern: /only/i,
    longer_alt: Identifier_1.default,
});
const WithTiesKw = (0, chevrotain_1.createToken)({
    name: 'WithTiesKw',
    pattern: /with\s+ties/i,
    longer_alt: Identifier_1.default,
});
const WithKw = (0, chevrotain_1.createToken)({
    name: 'WithKw',
    pattern: /with/i,
    longer_alt: [WithinGroupKw, WithTiesKw, Identifier_1.default],
});
const OnKw = (0, chevrotain_1.createToken)({
    name: 'OnKw',
    pattern: /on/i,
    longer_alt: [Identifier_1.default, OnlyKw],
});
const JsonTableKw = (0, chevrotain_1.createToken)({
    name: 'JsonTableKw',
    pattern: /json_table/i,
    longer_alt: Identifier_1.default,
});
const ColumnsKw = (0, chevrotain_1.createToken)({
    name: 'ColumnsKw',
    pattern: /columns/i,
    longer_alt: Identifier_1.default,
});
const PathKw = (0, chevrotain_1.createToken)({
    name: 'PathKw',
    pattern: /path/i,
    longer_alt: Identifier_1.default,
    categories: [specialIdentifiers_1.KwIdentifier],
});
exports.default = [
    SelectKw,
    IntoKw,
    FromKw,
    WhereKw,
    GroupByKw,
    OrderKw,
    AscDescKw,
    NullsFirstLastKw,
    Null,
    AndOr,
    AndKw,
    OrKw,
    JoinKw,
    JoinDirection,
    LeftKw,
    RightKw,
    InnerKw,
    OuterKw,
    CrossKw,
    InsertKw,
    ValuesKw,
    DeleteKw,
    UpdateKw,
    SetKw,
    HavingKw,
    OverKw,
    PartitionByKw,
    WithinGroupKw,
    FetchFirstLastKw,
    FetchKw,
    PercentKw,
    OnlyKw,
    WithTiesKw,
    WithKw,
    OnKw,
    JsonTableKw,
    ColumnsKw,
    PathKw,
];
//# sourceMappingURL=sql.js.map