"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chevrotain_1 = require("chevrotain");
const Identifier_1 = require("./Identifier");
const ValueSeperator = (0, chevrotain_1.createToken)({
    name: 'ValueSeperator',
    pattern: chevrotain_1.Lexer.NA,
});
const Assignment = (0, chevrotain_1.createToken)({
    name: 'Assignment',
    pattern: /:=/,
    longer_alt: Identifier_1.default,
});
const SingleLineComment = (0, chevrotain_1.createToken)({
    name: 'SingleLineComment',
    pattern: /--.*/,
    group: chevrotain_1.Lexer.SKIPPED,
});
const MultiLineComment = (0, chevrotain_1.createToken)({
    name: 'MultiLineComment',
    pattern: /\/\*[^`]*?\*\//,
    line_breaks: true,
    group: chevrotain_1.Lexer.SKIPPED,
});
const Semicolon = (0, chevrotain_1.createToken)({
    name: 'Semicolon',
    pattern: /;/,
    longer_alt: Identifier_1.default,
});
const Comma = (0, chevrotain_1.createToken)({
    name: 'Comma',
    pattern: /,/,
    longer_alt: Identifier_1.default,
});
const AdditionOperator = (0, chevrotain_1.createToken)({
    name: 'AdditionOperator',
    pattern: chevrotain_1.Lexer.NA,
    categories: ValueSeperator,
});
const Plus = (0, chevrotain_1.createToken)({
    name: 'Plus',
    pattern: /\+/,
    longer_alt: Identifier_1.default,
    categories: AdditionOperator,
});
const Minus = (0, chevrotain_1.createToken)({
    name: 'Minus',
    pattern: /-/,
    longer_alt: SingleLineComment,
    categories: AdditionOperator,
});
const MultiplicationOperator = (0, chevrotain_1.createToken)({
    name: 'MultiplicationOperator',
    pattern: chevrotain_1.Lexer.NA,
    categories: ValueSeperator,
});
const Asterisk = (0, chevrotain_1.createToken)({
    name: 'Asterisk',
    pattern: /\*/,
    longer_alt: Identifier_1.default,
    categories: MultiplicationOperator,
});
const Slash = (0, chevrotain_1.createToken)({
    name: 'Slash',
    pattern: /\//,
    longer_alt: Identifier_1.default,
    categories: MultiplicationOperator,
});
const OpenBracket = (0, chevrotain_1.createToken)({
    name: 'OpenBracket',
    pattern: /\(/,
    longer_alt: Identifier_1.default,
});
const ClosingBracket = (0, chevrotain_1.createToken)({
    name: 'ClosingBracket',
    pattern: /\)/,
    longer_alt: Identifier_1.default,
});
const Concat = (0, chevrotain_1.createToken)({
    name: 'Concat',
    pattern: /\|\|/,
    categories: ValueSeperator,
});
const Pipe = (0, chevrotain_1.createToken)({
    name: 'Pipe',
    pattern: /\|/,
    longer_alt: Concat,
});
const Percent = (0, chevrotain_1.createToken)({
    name: 'Percent',
    pattern: /%/,
    longer_alt: Identifier_1.default,
});
const CompileErrorKw = (0, chevrotain_1.createToken)({
    name: 'CompileErrorKw',
    pattern: /\$error/i,
    longer_alt: Identifier_1.default,
});
const Dollar = (0, chevrotain_1.createToken)({
    name: 'Dollar',
    pattern: /\$/,
    longer_alt: CompileErrorKw,
});
const Dot = (0, chevrotain_1.createToken)({
    name: 'Dot',
    pattern: /\./,
    longer_alt: Identifier_1.default,
});
const Arrow = (0, chevrotain_1.createToken)({
    name: 'Arrow',
    pattern: /=>/,
    longer_alt: Identifier_1.default,
});
exports.default = [
    Assignment,
    SingleLineComment,
    MultiLineComment,
    Semicolon,
    Comma,
    Plus,
    Minus,
    Asterisk,
    Slash,
    OpenBracket,
    ClosingBracket,
    Concat,
    Pipe,
    Percent,
    CompileErrorKw,
    Dollar,
    Dot,
    Arrow,
    AdditionOperator,
    MultiplicationOperator,
    ValueSeperator,
];
//# sourceMappingURL=symbols.js.map