"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const readFile_1 = require("./util/readFile");
const tokens_1 = require("./components/tokenDictionary/tokens");
const noRecoveryParser_1 = require("./components/mainParser/noRecoveryParser");
const cstVisitor_1 = require("./components/cstVisitor");
// const yellowLog = (text) => `\x1b[33m${text}\x1b[0m`;
function getText(token) {
    if (token.tokenType.name === 'MultiLineComment') {
        return '/* ... */';
    }
    if (token.image.length > 35) {
        return `${token.image.substr(0, 32)}...`;
    }
    return token.image;
}
const logLexer = (file) => {
    const lexRes = (0, tokens_1.lex)(file);
    const arr = [];
    lexRes.tokens.forEach((token) => {
        // let logText = `${yellowLog(token.image)}`;
        // logText += ` | Token: "${token.tokenType.name}"`;
        // if (token.tokenType?.LONGER_ALT?.name) {
        //   logText += ` - Longer Alt: "${token.tokenType?.LONGER_ALT?.name}"`;
        // }
        // console.log(logText);
        arr.push({
            text: getText(token),
            token: token.tokenType.name.length > 35
                ? `${token.tokenType.name.substr(0, 32)}...`
                : token.tokenType.name,
            longerAlt: token.tokenType?.LONGER_ALT?.name,
            position: `${token.startLine}:${token.startColumn} - ${token.endLine}:${token.endColumn}`,
        });
    });
    console.table(arr);
};
const main = async () => {
    try {
        const file = await (0, readFile_1.default)('./test/plsql/current-error.sql');
        if (typeof file !== 'string') {
            throw new Error('File is not a string');
        }
        logLexer(file);
        const res = (0, noRecoveryParser_1.default)(file, true);
        const interpreted = cstVisitor_1.default.visit(res.cst);
        console.log('interpreted', JSON.stringify(interpreted, null, 2));
    }
    catch (err) {
        console.log(err);
    }
};
main();
//# sourceMappingURL=dev-test-file.js.map